@@ -29,42 +29,26 @@
 #                 Frédéric Buclin <LpSolit@gmail.com>
 #
 
-# This file defines all the parameters that we have a GUI to edit within
-# Bugzilla.
-
-# ATTENTION!!!!   THIS FILE ONLY CONTAINS THE DEFAULTS.
-# You cannot change your live settings by editing this file.
-# Only adding new parameters is done here.  Once the parameter exists, you
-# must use %baseurl%/editparams.cgi from the web to edit the settings.
-
-# This file is included via |do|, mainly because of circular dependency issues
-# (such as globals.pl -> Bugzilla::Config -> this -> Bugzilla::Config)
-# which preclude compile time loading.
-
-# Those issues may go away at some point, and the contents of this file
-# moved somewhere else. Please try to avoid more dependencies from here
-# to other code
-
-# (Note that these aren't just added directly to Bugzilla::Config, because
-# the backend prefs code is separate to this...)
-
 package Bugzilla::Config::Common;
 
 use strict;
 
 use Socket;
+use Time::Zone;
 
-use Bugzilla::Config qw(:DEFAULT $templatedir $webdotdir);
 use Bugzilla::Util;
 use Bugzilla::Constants;
+use Bugzilla::Field;
+use Bugzilla::Group;
 
 use base qw(Exporter);
 @Bugzilla::Config::Common::EXPORT =
-    qw(check_multi check_numeric check_regexp
+    qw(check_multi check_numeric check_regexp check_url check_group
        check_sslbase check_priority check_severity check_platform
        check_opsys check_shadowdb check_urlbase check_webdotbase
        check_netmask check_user_verify_class check_image_converter
-       check_languages check_mail_delivery_method
+       check_languages check_mail_delivery_method check_notification
+       check_timezone check_utf8
 );
 
 # Checking functions for the various values
@@ -130,42 +114,64 @@ sub check_sslbase {
     return "";
 }
 
+sub check_utf8 {
+    my $utf8 = shift;
+    # You cannot turn off the UTF-8 parameter if you've already converted
+    # your tables to utf-8.
+    my $dbh = Bugzilla->dbh;
+    if ($dbh->isa('Bugzilla::DB::Mysql') && $dbh->bz_db_is_utf8 && !$utf8) {
+        return "You cannot disable UTF-8 support, because your MySQL database"
+               . " is encoded in UTF-8";
+    }
+    return "";
+}
+
 sub check_priority {
     my ($value) = (@_);
-    &::GetVersionTable();
-    if (lsearch(\@::legal_priority, $value) < 0) {
+    my $legal_priorities = get_legal_field_values('priority');
+    if (lsearch($legal_priorities, $value) < 0) {
         return "Must be a legal priority value: one of " .
-            join(", ", @::legal_priority);
+            join(", ", @$legal_priorities);
     }
     return "";
 }
 
 sub check_severity {
     my ($value) = (@_);
-    &::GetVersionTable();
-    if (lsearch(\@::legal_severity, $value) < 0) {
+    my $legal_severities = get_legal_field_values('bug_severity');
+    if (lsearch($legal_severities, $value) < 0) {
         return "Must be a legal severity value: one of " .
-            join(", ", @::legal_severity);
+            join(", ", @$legal_severities);
     }
     return "";
 }
 
 sub check_platform {
     my ($value) = (@_);
-    &::GetVersionTable();
-    if (lsearch(['', @::legal_platform], $value) < 0) {
+    my $legal_platforms = get_legal_field_values('rep_platform');
+    if (lsearch(['', @$legal_platforms], $value) < 0) {
         return "Must be empty or a legal platform value: one of " .
-            join(", ", @::legal_platform);
+            join(", ", @$legal_platforms);
     }
     return "";
 }
 
 sub check_opsys {
     my ($value) = (@_);
-    &::GetVersionTable();
-    if (lsearch(['', @::legal_opsys], $value) < 0) {
+    my $legal_OS = get_legal_field_values('op_sys');
+    if (lsearch(['', @$legal_OS], $value) < 0) {
         return "Must be empty or a legal operating system value: one of " .
-            join(", ", @::legal_opsys);
+            join(", ", @$legal_OS);
+    }
+    return "";
+}
+
+sub check_group {
+    my $group_name = shift;
+    return "" unless $group_name;
+    my $group = new Bugzilla::Group({'name' => $group_name});
+    unless (defined $group) {
+        return "Must be an existing group name";
     }
     return "";
 }
@@ -177,7 +183,7 @@ sub check_shadowdb {
         return "";
     }
 
-    if (!Param('shadowdbhost')) {
+    if (!Bugzilla->params->{'shadowdbhost'}) {
         return "You need to specify a host when using a shadow database";
     }
 
@@ -195,6 +201,15 @@ sub check_urlbase {
     return "";
 }
 
+sub check_url {
+    my ($url) = (@_);
+    return '' if $url eq ''; # Allow empty URLs
+    if ($url !~ m:/$:) {
+        return 'must be a legal URL, absolute or relative, ending with a slash.';
+    }
+    return '';
+}
+
 sub check_webdotbase {
     my ($value) = (@_);
     $value = trim($value);
@@ -206,6 +221,7 @@ sub check_webdotbase {
             return "The file path \"$value\" is not a valid executable.  Please specify the complete file path to 'dot' if you intend to generate graphs locally.";
         }
         # Check .htaccess allows access to generated images
+        my $webdotdir = bz_locations()->{'webdotdir'};
         if(-e "$webdotdir/.htaccess") {
             open HTACCESS, "$webdotdir/.htaccess";
             if(! grep(/ \\\.png\$/,<HTACCESS>)) {
@@ -227,7 +243,7 @@ sub check_netmask {
     # Note that if we changed the netmask from anything apart from 32, then
     # existing logincookies which aren't for a single IP won't work
     # any more. We can't know which ones they are, though, so they'll just
-    # take space until they're preiodically cleared, later.
+    # take space until they're periodically cleared, later.
 
     return "";
 }
@@ -250,8 +266,8 @@ sub check_user_verify_class {
         } elsif ($class eq 'LDAP') {
             eval "require Net::LDAP";
             return "Error requiring Net::LDAP: '$@'" if $@;
-            return "LDAP servername is missing" unless Param("LDAPserver");
-            return "LDAPBaseDN is empty" unless Param("LDAPBaseDN");
+            return "LDAP servername is missing" unless Bugzilla->params->{"LDAPserver"};
+            return "LDAPBaseDN is empty" unless Bugzilla->params->{"LDAPBaseDN"};
         } else {
                 return "Unknown user_verify_class '$class' in check_user_verify_class";
         }
@@ -269,16 +285,26 @@ sub check_image_converter {
 }
 
 sub check_languages {
-    my @languages = split /[,\s]+/, trim($_[0]);
+    my ($lang, $param) = @_;
+    my @languages = split(/[,\s]+/, trim($lang));
     if(!scalar(@languages)) {
        return "You need to specify a language tag."
     }
+    if (scalar(@languages) > 1 && $param && $param->{'name'} eq 'defaultlanguage') {
+        return "You can only specify one language tag";
+    }
+    my $templatedir = bz_locations()->{'templatedir'};
+    my %lang_seen;
+    my @validated_languages;
     foreach my $language (@languages) {
        if(   ! -d "$templatedir/$language/custom" 
           && ! -d "$templatedir/$language/default") {
           return "The template directory for $language does not exist";
        }
+       push(@validated_languages, $language) unless $lang_seen{$language}++;
     }
+    # Rebuild the list of language tags, avoiding duplicates.
+    $_[0] = join(', ', @validated_languages);
     return "";
 }
 
@@ -294,6 +320,28 @@ sub check_mail_delivery_method {
     return "";
 }
 
+sub check_notification {
+    my $option = shift;
+    my @current_version =
+        (BUGZILLA_VERSION =~ m/^(\d+)\.(\d+)(?:(rc|\.)(\d+))?\+?$/);
+    if ($current_version[1] % 2 && $option eq 'stable_branch_release') {
+        return "You are currently running a development snapshot, and so your " .
+               "installation is not based on a branch. If you want to be notified " .
+               "about the next stable release, you should select " .
+               "'latest_stable_release' instead";
+    }
+    return "";
+}
+
+sub check_timezone {
+    my $tz = shift;
+    unless (defined(tz_offset($tz))) {
+        return "must be empty or a legal timezone name, such as PDT or JST";
+    }
+    return "";
+}
+
+
 # OK, here are the parameter definitions themselves.
 #
 # Each definition is a hash with keys:
