@@ -22,6 +22,7 @@
 #                 Max Kanat-Alexander <mkanat@bugzilla.org>
 #                 Lance Larsh <lance.larsh@oracle.com>
 #                 Dennis Melentyev <dennis.melentyev@infopulse.com.ua>
+#                 Akamai Technologies <bugzilla-dev@akamai.com>
 
 package Bugzilla::DB::Schema;
 
@@ -35,8 +36,11 @@ package Bugzilla::DB::Schema;
 
 use strict;
 use Bugzilla::Error;
+use Bugzilla::Hook;
 use Bugzilla::Util;
+use Bugzilla::Constants;
 
+use Hash::Util qw(lock_value unlock_hash lock_keys unlock_keys);
 use Safe;
 # Historical, needed for SCHEMA_VERSION = '1.00'
 use Storable qw(dclone freeze thaw);
@@ -78,6 +82,13 @@ That means that CGI scripts should never call any function in this
 module directly, but should instead rely on methods provided by
 Bugzilla::DB.
 
+=head1 NEW TO SCHEMA.PM?
+
+If this is your first time looking at Schema.pm, especially if
+you are making changes to the database, please take a look at
+L<http://www.bugzilla.org/docs/developer.html#sql-schema> to learn
+more about how this integrates into the rest of Bugzilla.
+
 =cut
 
 #--------------------------------------------------------------------------
@@ -166,7 +177,7 @@ use constant ABSTRACT_SCHEMA => {
             bug_status          => {TYPE => 'varchar(64)', NOTNULL => 1},
             creation_ts         => {TYPE => 'DATETIME'},
             delta_ts            => {TYPE => 'DATETIME', NOTNULL => 1},
-            short_desc          => {TYPE => 'MEDIUMTEXT', NOTNULL => 1},
+            short_desc          => {TYPE => 'varchar(255)', NOTNULL => 1},
             op_sys              => {TYPE => 'varchar(64)', NOTNULL => 1},
             priority            => {TYPE => 'varchar(64)', NOTNULL => 1},
             product_id          => {TYPE => 'INT2', NOTNULL => 1},
@@ -218,8 +229,6 @@ use constant ABSTRACT_SCHEMA => {
             bugs_target_milestone_idx => ['target_milestone'],
             bugs_qa_contact_idx       => ['qa_contact'],
             bugs_votes_idx            => ['votes'],
-            bugs_short_desc_idx       => {FIELDS => ['short_desc'],
-                                          TYPE => 'FULLTEXT'},
         ],
     },
 
@@ -255,20 +264,25 @@ use constant ABSTRACT_SCHEMA => {
 
     longdescs => {
         FIELDS => [
+            comment_id      => {TYPE => 'MEDIUMSERIAL',  NOTNULL => 1,
+                                PRIMARYKEY => 1},
             bug_id          => {TYPE => 'INT3',  NOTNULL => 1},
             who             => {TYPE => 'INT3', NOTNULL => 1},
             bug_when        => {TYPE => 'DATETIME', NOTNULL => 1},
             work_time       => {TYPE => 'decimal(5,2)', NOTNULL => 1,
                                 DEFAULT => '0'},
-            thetext         => {TYPE => 'MEDIUMTEXT'},
+            thetext         => {TYPE => 'MEDIUMTEXT', NOTNULL => 1},
             isprivate       => {TYPE => 'BOOLEAN', NOTNULL => 1,
                                 DEFAULT => 'FALSE'},
             already_wrapped => {TYPE => 'BOOLEAN', NOTNULL => 1,
                                 DEFAULT => 'FALSE'},
+            type            => {TYPE => 'INT2', NOTNULL => 1,
+                                DEFAULT => '0'},
+            extra_data      => {TYPE => 'varchar(255)'}
         ],
         INDEXES => [
             longdescs_bug_id_idx   => ['bug_id'],
-            longdescs_who_idx     => ['who'],
+            longdescs_who_idx     => [qw(who bug_id)],
             longdescs_bug_when_idx => ['bug_when'],
             longdescs_thetext_idx => {FIELDS => ['thetext'],
                                       TYPE => 'FULLTEXT'},
@@ -343,7 +357,7 @@ use constant ABSTRACT_SCHEMA => {
 
     keyworddefs => {
         FIELDS => [
-            id          => {TYPE => 'INT2', NOTNULL => 1,
+            id          => {TYPE => 'SMALLSERIAL', NOTNULL => 1,
                             PRIMARYKEY => 1},
             name        => {TYPE => 'varchar(64)', NOTNULL => 1},
             description => {TYPE => 'MEDIUMTEXT'},
@@ -372,7 +386,7 @@ use constant ABSTRACT_SCHEMA => {
     # "flags" stores one record for each flag on each bug/attachment.
     flags => {
         FIELDS => [
-            id                => {TYPE => 'INT3', NOTNULL => 1,
+            id                => {TYPE => 'MEDIUMSERIAL', NOTNULL => 1,
                                   PRIMARYKEY => 1},
             type_id           => {TYPE => 'INT2', NOTNULL => 1},
             status            => {TYPE => 'char(1)', NOTNULL => 1},
@@ -382,8 +396,6 @@ use constant ABSTRACT_SCHEMA => {
             modification_date => {TYPE => 'DATETIME'},
             setter_id         => {TYPE => 'INT3'},
             requestee_id      => {TYPE => 'INT3'},
-            is_active         => {TYPE => 'BOOLEAN', NOTNULL => 1,
-                                  DEFAULT => 'TRUE'},
         ],
         INDEXES => [
             flags_bug_id_idx       => [qw(bug_id attach_id)],
@@ -396,7 +408,7 @@ use constant ABSTRACT_SCHEMA => {
     # "flagtypes" defines the types of flags that can be set.
     flagtypes => {
         FIELDS => [
-            id               => {TYPE => 'INT2', NOTNULL => 1,
+            id               => {TYPE => 'SMALLSERIAL', NOTNULL => 1,
                                  PRIMARYKEY => 1},
             name             => {TYPE => 'varchar(50)', NOTNULL => 1},
             description      => {TYPE => 'TEXT'},
@@ -450,15 +462,21 @@ use constant ABSTRACT_SCHEMA => {
 
     fielddefs => {
         FIELDS => [
-            fieldid     => {TYPE => 'MEDIUMSERIAL', NOTNULL => 1,
+            id          => {TYPE => 'MEDIUMSERIAL', NOTNULL => 1,
                             PRIMARYKEY => 1},
             name        => {TYPE => 'varchar(64)', NOTNULL => 1},
+            type        => {TYPE => 'INT2', NOTNULL => 1,
+                            DEFAULT => FIELD_TYPE_UNKNOWN},
+            custom      => {TYPE => 'BOOLEAN', NOTNULL => 1,
+                            DEFAULT => 'FALSE'},
             description => {TYPE => 'MEDIUMTEXT', NOTNULL => 1},
             mailhead    => {TYPE => 'BOOLEAN', NOTNULL => 1,
                             DEFAULT => 'FALSE'},
             sortkey     => {TYPE => 'INT2', NOTNULL => 1},
             obsolete    => {TYPE => 'BOOLEAN', NOTNULL => 1,
                             DEFAULT => 'FALSE'},
+            enter_bug   => {TYPE => 'BOOLEAN', NOTNULL => 1,
+                            DEFAULT => 'FALSE'},
         ],
         INDEXES => [
             fielddefs_name_idx    => {FIELDS => ['name'],
@@ -472,6 +490,8 @@ use constant ABSTRACT_SCHEMA => {
 
     versions => {
         FIELDS => [
+            id         =>  {TYPE => 'MEDIUMSERIAL', NOTNULL => 1,
+                            PRIMARYKEY => 1},
             value      =>  {TYPE => 'varchar(64)', NOTNULL => 1},
             product_id =>  {TYPE => 'INT2', NOTNULL => 1},
         ],
@@ -483,6 +503,8 @@ use constant ABSTRACT_SCHEMA => {
 
     milestones => {
         FIELDS => [
+            id         => {TYPE => 'MEDIUMSERIAL', NOTNULL => 1, 
+                           PRIMARYKEY => 1},
             product_id => {TYPE => 'INT2', NOTNULL => 1},
             value      => {TYPE => 'varchar(20)', NOTNULL => 1},
             sortkey    => {TYPE => 'INT2', NOTNULL => 1,
@@ -605,11 +627,14 @@ use constant ABSTRACT_SCHEMA => {
                                PRIMARYKEY => 1},
             login_name     => {TYPE => 'varchar(255)', NOTNULL => 1},
             cryptpassword  => {TYPE => 'varchar(128)'},
-            realname       => {TYPE => 'varchar(255)'},
-            disabledtext   => {TYPE => 'MEDIUMTEXT', NOTNULL => 1},
+            realname       => {TYPE => 'varchar(255)', NOTNULL => 1,
+                               DEFAULT => "''"},
+            disabledtext   => {TYPE => 'MEDIUMTEXT', NOTNULL => 1,
+                               DEFAULT => "''"},
+            disable_mail   => {TYPE => 'BOOLEAN', NOTNULL => 1,
+                               DEFAULT => 'FALSE'},
             mybugslink     => {TYPE => 'BOOLEAN', NOTNULL => 1,
                                DEFAULT => 'TRUE'},
-            refreshed_when => {TYPE => 'DATETIME', NOTNULL => 1},
             extern_id      => {TYPE => 'varchar(64)'},
         ],
         INDEXES => [
@@ -661,9 +686,10 @@ use constant ABSTRACT_SCHEMA => {
 
     namedqueries => {
         FIELDS => [
+            id           => {TYPE => 'MEDIUMSERIAL', NOTNULL => 1,
+                             PRIMARYKEY => 1},
             userid       => {TYPE => 'INT3', NOTNULL => 1},
             name         => {TYPE => 'varchar(64)', NOTNULL => 1},
-            linkinfooter => {TYPE => 'BOOLEAN', NOTNULL => 1},
             query        => {TYPE => 'MEDIUMTEXT', NOTNULL => 1},
             query_type   => {TYPE => 'BOOLEAN', NOTNULL => 1},
         ],
@@ -673,6 +699,30 @@ use constant ABSTRACT_SCHEMA => {
         ],
     },
 
+    namedqueries_link_in_footer => {
+        FIELDS => [
+            namedquery_id => {TYPE => 'INT3', NOTNULL => 1},
+            user_id       => {TYPE => 'INT3', NOTNULL => 1},
+        ],
+        INDEXES => [
+            namedqueries_link_in_footer_id_idx => {FIELDS => [qw(namedquery_id user_id)],
+                                                   TYPE => 'UNIQUE'},
+            namedqueries_link_in_footer_userid_idx => ['user_id'],
+        ],
+    },
+
+    component_cc => {
+
+        FIELDS => [
+            user_id      => {TYPE => 'INT3', NOTNULL => 1},
+            component_id => {TYPE => 'INT2', NOTNULL => 1},
+        ],
+        INDEXES => [
+            component_cc_user_id_idx => {FIELDS => [qw(component_id user_id)],
+                                         TYPE => 'UNIQUE'},
+        ],
+    },
+
     # Authentication
     # --------------
 
@@ -694,7 +744,7 @@ use constant ABSTRACT_SCHEMA => {
     #     for these changes.
     tokens => {
         FIELDS => [
-            userid    => {TYPE => 'INT3', NOTNULL => 1} ,
+            userid    => {TYPE => 'INT3'},
             issuedate => {TYPE => 'DATETIME', NOTNULL => 1} ,
             token     => {TYPE => 'varchar(16)', NOTNULL => 1,
                           PRIMARYKEY => 1},
@@ -716,7 +766,6 @@ use constant ABSTRACT_SCHEMA => {
             name         => {TYPE => 'varchar(255)', NOTNULL => 1},
             description  => {TYPE => 'TEXT', NOTNULL => 1},
             isbuggroup   => {TYPE => 'BOOLEAN', NOTNULL => 1},
-            last_changed => {TYPE => 'DATETIME', NOTNULL => 1},
             userregexp   => {TYPE => 'TINYTEXT', NOTNULL => 1,
                              DEFAULT => "''"},
             isactive     => {TYPE => 'BOOLEAN', NOTNULL => 1,
@@ -735,6 +784,12 @@ use constant ABSTRACT_SCHEMA => {
             membercontrol => {TYPE => 'BOOLEAN', NOTNULL => 1},
             othercontrol  => {TYPE => 'BOOLEAN', NOTNULL => 1},
             canedit       => {TYPE => 'BOOLEAN', NOTNULL => 1},
+            editcomponents => {TYPE => 'BOOLEAN', NOTNULL => 1,
+                               DEFAULT => 'FALSE'},
+            editbugs      => {TYPE => 'BOOLEAN', NOTNULL => 1,
+                              DEFAULT => 'FALSE'},
+            canconfirm    => {TYPE => 'BOOLEAN', NOTNULL => 1,
+                              DEFAULT => 'FALSE'},
         ],
         INDEXES => [
             group_control_map_product_id_idx =>
@@ -757,7 +812,7 @@ use constant ABSTRACT_SCHEMA => {
             isbless    => {TYPE => 'BOOLEAN', NOTNULL => 1,
                            DEFAULT => 'FALSE'},
             grant_type => {TYPE => 'INT1', NOTNULL => 1,
-                           DEFAULT => '0'},
+                           DEFAULT => GRANT_DIRECT},
         ],
         INDEXES => [
             user_group_map_user_id_idx =>
@@ -778,7 +833,7 @@ use constant ABSTRACT_SCHEMA => {
             member_id  => {TYPE => 'INT3', NOTNULL => 1},
             grantor_id => {TYPE => 'INT3', NOTNULL => 1},
             grant_type => {TYPE => 'INT1', NOTNULL => 1,
-                           DEFAULT => '0'},
+                           DEFAULT => GROUP_MEMBERSHIP},
         ],
         INDEXES => [
             group_group_map_member_id_idx =>
@@ -801,6 +856,20 @@ use constant ABSTRACT_SCHEMA => {
         ],
     },
 
+    # This table determines which groups a user must be a member of
+    # in order to see a named query somebody else shares.
+    namedquery_group_map => {
+        FIELDS => [
+            namedquery_id => {TYPE => 'INT3', NOTNULL => 1},
+            group_id      => {TYPE => 'INT3', NOTNULL => 1},
+        ],
+        INDEXES => [
+            namedquery_group_map_namedquery_id_idx   =>
+                {FIELDS => [qw(namedquery_id)], TYPE => 'UNIQUE'},
+            namedquery_group_map_group_id_idx => ['group_id'],
+        ],
+    },
+
     category_group_map => {
         FIELDS => [
             category_id => {TYPE => 'INT2', NOTNULL => 1},
@@ -822,6 +891,7 @@ use constant ABSTRACT_SCHEMA => {
                             PRIMARYKEY => 1},
             name        => {TYPE => 'varchar(64)', NOTNULL => 1},
             description => {TYPE => 'MEDIUMTEXT'},
+            sortkey     => {TYPE => 'INT2', NOTNULL => 1, DEFAULT => '0'},
         ],
         INDEXES => [
             classifications_name_idx => {FIELDS => ['name'],
@@ -837,12 +907,16 @@ use constant ABSTRACT_SCHEMA => {
             classification_id => {TYPE => 'INT2', NOTNULL => 1,
                                   DEFAULT => '1'},
             description       => {TYPE => 'MEDIUMTEXT'},
-            milestoneurl      => {TYPE => 'TINYTEXT', NOTNULL => 1},
-            disallownew       => {TYPE => 'BOOLEAN', NOTNULL => 1},
-            votesperuser      => {TYPE => 'INT2', NOTNULL => 1},
+            milestoneurl      => {TYPE => 'TINYTEXT', NOTNULL => 1,
+                                  DEFAULT => "''"},
+            disallownew       => {TYPE => 'BOOLEAN', NOTNULL => 1,
+                                  DEFAULT => 0},
+            votesperuser      => {TYPE => 'INT2', NOTNULL => 1,
+                                  DEFAULT => 0},
             maxvotesperbug    => {TYPE => 'INT2', NOTNULL => 1,
                                   DEFAULT => '10000'},
-            votestoconfirm    => {TYPE => 'INT2', NOTNULL => 1},
+            votestoconfirm    => {TYPE => 'INT2', NOTNULL => 1,
+                                  DEFAULT => 0},
             defaultmilestone  => {TYPE => 'varchar(20)',
                                   NOTNULL => 1, DEFAULT => "'---'"},
         ],
@@ -876,7 +950,7 @@ use constant ABSTRACT_SCHEMA => {
         FIELDS => [
             series_id   => {TYPE => 'MEDIUMSERIAL', NOTNULL => 1,
                             PRIMARYKEY => 1},
-            creator     => {TYPE => 'INT3', NOTNULL => 1},
+            creator     => {TYPE => 'INT3'},
             category    => {TYPE => 'INT2', NOTNULL => 1},
             subcategory => {TYPE => 'INT2', NOTNULL => 1},
             name        => {TYPE => 'varchar(64)', NOTNULL => 1},
@@ -1001,6 +1075,7 @@ use constant ABSTRACT_SCHEMA => {
             default_value => {TYPE => 'varchar(32)', NOTNULL => 1},
             is_enabled    => {TYPE => 'BOOLEAN', NOTNULL => 1,
                               DEFAULT => 'TRUE'},
+            subclass      => {TYPE => 'varchar(32)'},
         ],
     },
 
@@ -1041,12 +1116,29 @@ use constant ABSTRACT_SCHEMA => {
     },
 
 };
+
+use constant FIELD_TABLE_SCHEMA => {
+    FIELDS => [
+        id       => {TYPE => 'SMALLSERIAL', NOTNULL => 1,
+                     PRIMARYKEY => 1},
+        value    => {TYPE => 'varchar(64)', NOTNULL => 1},
+        sortkey  => {TYPE => 'INT2', NOTNULL => 1, DEFAULT => 0},
+        isactive => {TYPE => 'BOOLEAN', NOTNULL => 1,
+                     DEFAULT => 'TRUE'},
+    ],
+    # Note that bz_add_field_table should prepend the table name
+    # to these index names.
+    INDEXES => [
+        value_idx   => {FIELDS => ['value'], TYPE => 'UNIQUE'},
+        sortkey_idx => ['sortkey', 'value'],
+    ],
+};
 #--------------------------------------------------------------------------
 
 =head1 METHODS
 
 Note: Methods which can be implemented generically for all DBs are
-implemented in this module. If needed, they can be overriden with
+implemented in this module. If needed, they can be overridden with
 DB-specific code in a subclass. Methods which are prefixed with C<_>
 are considered protected. Subclasses may override these methods, but
 other modules should not invoke these methods directly.
@@ -1119,13 +1211,26 @@ sub _initialize {
     my $self = shift;
     my $abstract_schema = shift;
 
-    $abstract_schema ||= ABSTRACT_SCHEMA;
+    if (!$abstract_schema) {
+        # While ABSTRACT_SCHEMA cannot be modified, $abstract_schema can be.
+        # So, we dclone it to prevent anything from mucking with the constant.
+        $abstract_schema = dclone(ABSTRACT_SCHEMA);
+
+        # Let extensions add tables, but make sure they can't modify existing
+        # tables. If we don't lock/unlock keys, lock_value complains.
+        lock_keys(%$abstract_schema);
+        foreach my $table (keys %{ABSTRACT_SCHEMA()}) {
+            lock_value(%$abstract_schema, $table) 
+                if exists $abstract_schema->{$table};
+        }
+        unlock_keys(%$abstract_schema);
+        Bugzilla::Hook::process('db_schema-abstract_schema', 
+                                { schema => $abstract_schema });
+        unlock_hash(%$abstract_schema);
+    }
 
     $self->{schema} = dclone($abstract_schema);
-    # While ABSTRACT_SCHEMA cannot be modified, 
-    # $self->{abstract_schema} can be. So, we dclone it to prevent
-    # anything from mucking with the constant.
-    $self->{abstract_schema} = dclone($abstract_schema);
+    $self->{abstract_schema} = $abstract_schema;
 
     return $self;
 
@@ -1155,7 +1260,7 @@ sub _adjust_schema {
     # Loop over each table in the abstract database schema.
     foreach my $table (keys %{ $self->{schema} }) {
         my %fields = (@{ $self->{schema}{$table}{FIELDS} });
-        # Loop over the field defintions in each table.
+        # Loop over the field definitions in each table.
         foreach my $field_def (values %fields) {
             # If the field type is an abstract data type defined in the
             # $db_specific hash, replace it with the DBMS-specific data type
@@ -1208,7 +1313,7 @@ sub get_type_ddl {
     }
 
     my $fkref = $self->{enable_references} ? $finfo->{REFERENCES} : undef;
-    my $type_ddl = $self->{db_specific}{$type} || $type;
+    my $type_ddl = $self->convert_type($type);
     # DEFAULT attribute must appear before any column constraints
     # (e.g., NOT NULL), for Oracle
     $type_ddl .= " DEFAULT $default" if (defined($default));
@@ -1219,7 +1324,19 @@ sub get_type_ddl {
     return($type_ddl);
 
 } #eosub--get_type_ddl
-#--------------------------------------------------------------------------
+
+sub convert_type {
+
+=item C<convert_type>
+
+Converts a TYPE from the L</ABSTRACT_SCHEMA> format into the real SQL type.
+
+=cut
+
+    my ($self, $type) = @_;
+    return $self->{db_specific}->{$type} || $type;
+}
+
 sub get_column {
 =item C<get_column($table, $column)>
 
@@ -1289,7 +1406,19 @@ sub get_table_columns {
     return @columns;
 
 } #eosub--get_table_columns
-#--------------------------------------------------------------------------
+
+sub get_table_indexes_abstract {
+    my ($self, $table) = @_;
+    my $table_def = $self->get_table_abstract($table);
+    my %indexes = @{$table_def->{INDEXES} || []};
+    return \%indexes;
+}
+
+sub get_create_database_sql {
+    my ($self, $name) = @_;
+    return ("CREATE DATABASE $name");
+}
+
 sub get_table_ddl {
 
 =item C<get_table_ddl>
@@ -1377,10 +1506,10 @@ sub _get_create_index_ddl {
 
 =cut
 
-    my($self, $table_name, $index_name, $index_fields, $index_type) = @_;
+    my ($self, $table_name, $index_name, $index_fields, $index_type) = @_;
 
     my $sql = "CREATE ";
-    $sql .= "$index_type " if ($index_type eq 'UNIQUE');
+    $sql .= "$index_type " if ($index_type && $index_type eq 'UNIQUE');
     $sql .= "INDEX $index_name ON $table_name \(" .
       join(", ", @$index_fields) . "\)";
 
@@ -1478,43 +1607,10 @@ sub get_alter_column_ddl {
     my $old_def = $self->get_column_abstract($table, $column);
     my $specific = $self->{db_specific};
 
-    my $typechange = 0;
     # If the types have changed, we have to deal with that.
     if (uc(trim($old_def->{TYPE})) ne uc(trim($new_def->{TYPE}))) {
-        $typechange = 1;
-        my $type = $new_def->{TYPE};
-        $type = $specific->{$type} if exists $specific->{$type};
-        # Make sure we can CAST from the old type to the new without an error.
-        push(@statements, "SELECT CAST($column AS $type) FROM $table LIMIT 1");
-        # Add a new temporary column of the new type
-        push(@statements, "ALTER TABLE $table ADD COLUMN ${column}_ALTERTEMP"
-                        . " $type");
-        # UPDATE the temp column to have the same values as the old column
-        push(@statements, "UPDATE $table SET ${column}_ALTERTEMP = " 
-                        . " CAST($column AS $type)");
-
-        # Some databases drop a whole index when a column is dropped,
-        # some only remove that column from an index. For consistency,
-        # we manually drop all indexes on the column before dropping the
-        # column.
-        my %col_idx = $self->get_indexes_on_column_abstract($table, $column);
-        foreach my $idx_name (keys %col_idx) {
-            push(@statements, $self->get_drop_index_ddl($table, $idx_name));
-        }
-
-        # DROP the old column
-        push(@statements, "ALTER TABLE $table DROP COLUMN $column");
-        # And rename the temp column to be the new one.
-        push(@statements, "ALTER TABLE $table RENAME COLUMN "
-                        . " ${column}_ALTERTEMP TO $column");
-
-        # And now, we have to regenerate any indexes that got
-        # dropped, except for the PK index which will be handled
-        # below.
-        foreach my $idx_name (keys %col_idx) {
-            push(@statements,
-                 $self->get_add_index_ddl($table, $idx_name, $col_idx{$idx_name}));
-        }
+        push(@statements, $self->_get_alter_type_sql($table, $column, 
+                                                     $new_def, $old_def));
     }
 
     my $default = $new_def->{DEFAULT};
@@ -1528,21 +1624,17 @@ sub get_alter_column_ddl {
         push(@statements, "ALTER TABLE $table ALTER COLUMN $column"
                         . " DROP DEFAULT");
     }
-    # If we went from no default to a default, or we changed the default,
-    # or we have a default and we changed the data type of the field
+    # If we went from no default to a default, or we changed the default.
     elsif ( (defined $default && !defined $default_old) || 
-         ($default ne $default_old) ||
-         ($typechange && defined $new_def->{DEFAULT}) ) {
+            ($default ne $default_old) ) 
+    {
         $default = $specific->{$default} if exists $specific->{$default};
         push(@statements, "ALTER TABLE $table ALTER COLUMN $column "
                          . " SET DEFAULT $default");
     }
 
-    # If we went from NULL to NOT NULL
-    # OR if we changed the type and we are NOT NULL
-    if ( (!$old_def->{NOTNULL} && $new_def->{NOTNULL}) ||
-         ($typechange && $new_def->{NOTNULL}) ) 
-    {
+    # If we went from NULL to NOT NULL.
+    if (!$old_def->{NOTNULL} && $new_def->{NOTNULL}) {
         my $setdefault;
         # Handle any fields that were NULL before, if we have a default,
         $setdefault = $new_def->{DEFAULT} if exists $new_def->{DEFAULT};
@@ -1563,10 +1655,8 @@ sub get_alter_column_ddl {
                         . " DROP NOT NULL");
     }
 
-    # If we went from not being a PRIMARY KEY to being a PRIMARY KEY,
-    # or if we changed types and we are a PK.
-    if ( (!$old_def->{PRIMARYKEY} && $new_def->{PRIMARYKEY}) ||
-         ($typechange && $new_def->{PRIMARYKEY}) ) {
+    # If we went from not being a PRIMARY KEY to being a PRIMARY KEY.
+    if (!$old_def->{PRIMARYKEY} && $new_def->{PRIMARYKEY}) {
         push(@statements, "ALTER TABLE $table ADD PRIMARY KEY ($column)");
     }
     # If we went from being a PK to not being a PK
@@ -1642,6 +1732,37 @@ sub get_rename_column_ddl {
         . " has not implemented a method.";
 }
 
+
+sub get_rename_table_sql {
+
+=item C<get_rename_table_sql>
+
+=over
+
+=item B<Description>
+
+Gets SQL to rename a table in the database.
+
+=item B<Params>
+
+=over
+
+=item C<$old_name> - The current name of the table.
+
+=item C<$new_name> - The new name of the table.
+
+=back
+
+=item B<Returns>: An array of SQL statements to rename a table.
+
+=back
+
+=cut
+
+    my ($self, $old_name, $new_name) = @_;
+    return ("ALTER TABLE $old_name RENAME TO $new_name");
+}
+
 =item C<delete_table($name)>
 
  Description: Deletes a table from this Schema object.
@@ -1793,6 +1914,23 @@ sub add_table {
     }
 }
 
+
+
+sub rename_table {
+
+=item C<rename_table>
+
+Renames a table from C<$old_name> to C<$new_name> in this Schema object.
+
+=cut
+
+
+    my ($self, $old_name, $new_name) = @_;
+    my $table = $self->get_table_abstract($old_name);
+    $self->delete_table($old_name);
+    $self->add_table($new_name, $table);
+}
+
 sub delete_column {
 
 =item C<delete_column($table, $column)>
@@ -2072,41 +2210,80 @@ __END__
 
 =head1 ABSTRACT DATA TYPES
 
-The following abstract data types are used:
+The size and range data provided here is only
+intended as a guide.  See your database's Bugzilla
+module (in this directory) for the most up-to-date
+values for these data types.  The following
+abstract data types are used:
 
 =over 4
 
 =item C<BOOLEAN>
 
+Logical value 0 or 1 where 1 is true, 0 is false.
+
 =item C<INT1>
 
+Integer values (-128 - 127 or 0 - 255 unsigned).
+
 =item C<INT2>
 
+Integer values (-32,768 - 32767 or 0 - 65,535 unsigned).
+
 =item C<INT3>
 
+Integer values (-8,388,608 - 8,388,607 or 0 - 16,777,215 unsigned)
+
 =item C<INT4>
 
+Integer values (-2,147,483,648 - 2,147,483,647 or 0 - 4,294,967,295 
+unsigned)
+
 =item C<SMALLSERIAL>
 
+An auto-increment L</INT1>
+
 =item C<MEDIUMSERIAL>
 
+An auto-increment L</INT3>
+
 =item C<INTSERIAL>
 
+An auto-increment L</INT4>
+
 =item C<TINYTEXT>
 
+Variable length string of characters up to 255 (2^8 - 1) characters wide 
+or more depending on the character set used.
+
 =item C<MEDIUMTEXT>
 
+Variable length string of characters up to 16M (2^24 - 1) characters wide 
+or more depending on the character set used.
+
 =item C<TEXT>
 
+Variable length string of characters up to 64K (2^16 - 1) characters wide 
+or more depending on the character set used.
+
 =item C<LONGBLOB>
 
+Variable length string of binary data up to 4M (2^32 - 1) bytes wide
+
 =item C<DATETIME>
 
+DATETIME support varies from database to database, however, it's generally 
+safe to say that DATETIME entries support all date/time combinations greater
+than 1900-01-01 00:00:00.  Note that the format used is C<YYYY-MM-DD hh:mm:ss>
+to be safe, though it's possible that your database may not require
+leading zeros.  For greatest compatibility, however, please make sure dates 
+are formatted as above for queries to guarantee consistent results.
+
 =back
 
 Database-specific subclasses should define the implementation for these data
 types as a hash reference stored internally in the schema object as
-C<db_specific>. This is typically done in overriden L<_initialize> method.
+C<db_specific>. This is typically done in overridden L<_initialize> method.
 
 The following abstract boolean values should also be defined on a
 database-specific basis:
@@ -2123,4 +2300,6 @@ database-specific basis:
 
 L<Bugzilla::DB>
 
+L<http://www.bugzilla.org/docs/developer.html#sql-schema>
+
 =cut
