@@ -18,6 +18,8 @@
 # Rights Reserved.
 #
 # Contributor(s): Bradley Baetz <bbaetz@acm.org>
+#                 Marc Schumann <wurblzap@gmail.com>
+#                 Frédéric Buclin <LpSolit@gmail.com>
 
 package Bugzilla::Error;
 
@@ -26,11 +28,22 @@ use base qw(Exporter);
 
 @Bugzilla::Error::EXPORT = qw(ThrowCodeError ThrowTemplateError ThrowUserError);
 
-use Bugzilla::Config qw($datadir);
 use Bugzilla::Constants;
+use Bugzilla::WebService::Constants;
 use Bugzilla::Util;
 use Date::Format;
 
+# We cannot use $^S to detect if we are in an eval(), because mod_perl
+# already eval'uates everything, so $^S = 1 in all cases under mod_perl!
+sub _in_eval {
+    my $in_eval = 0;
+    for (my $stack = 1; my $sub = (caller($stack))[3]; $stack++) {
+        last if $sub =~ /^ModPerl/;
+        $in_eval = 1 if $sub =~ /^\(eval\)/;
+    }
+    return $in_eval;
+}
+
 sub _throw_error {
     my ($name, $error, $vars) = @_;
 
@@ -40,8 +53,11 @@ sub _throw_error {
 
     # Make sure any locked tables are unlocked
     # and the transaction is rolled back (if supported)
-    Bugzilla->dbh->bz_unlock_tables(UNLOCK_ABORT);
+    # If we are within an eval(), do not unlock tables as we are
+    # eval'uating some test on purpose.
+    Bugzilla->dbh->bz_unlock_tables(UNLOCK_ABORT) unless _in_eval();
 
+    my $datadir = bz_locations()->{'datadir'};
     # If a writable $datadir/errorlog exists, log error details there.
     if (-w "$datadir/errorlog") {
         require Data::Dumper;
@@ -74,16 +90,27 @@ sub _throw_error {
     }
 
     my $template = Bugzilla->template;
-    if (Bugzilla->batch) {
-        my $message;
-        $template->process($name, $vars, \$message)
-          || ThrowTemplateError($template->error());
-        die("$message");
-    } else {
+    if (Bugzilla->error_mode == ERROR_MODE_WEBPAGE) {
         print Bugzilla->cgi->header();
         $template->process($name, $vars)
           || ThrowTemplateError($template->error());
     }
+    else {
+        my $message;
+        $template->process($name, $vars, \$message)
+          || ThrowTemplateError($template->error());
+        if (Bugzilla->error_mode == ERROR_MODE_DIE) {
+            die("$message\n");
+        }
+        elsif (Bugzilla->error_mode == ERROR_MODE_DIE_SOAP_FAULT) {
+            my $code = WS_ERROR_CODE->{$error};
+            if (!$code) {
+                $code = ERROR_UNKNOWN_FATAL if $name =~ /code/i;
+                $code = ERROR_UNKNOWN_TRANSIENT if $name =~ /user/i;
+            }
+            die SOAP::Fault->faultcode($code)->faultstring($message);
+        }
+    }
     exit;
 }
 
@@ -103,7 +130,7 @@ sub ThrowTemplateError {
     Bugzilla->dbh->bz_unlock_tables(UNLOCK_ABORT);
 
     my $vars = {};
-    if (Bugzilla->batch) {
+    if (Bugzilla->error_mode == ERROR_MODE_DIE) {
         die("error: template error: $template_err");
     }
 
@@ -115,9 +142,9 @@ sub ThrowTemplateError {
     # Try a template first; but if this one fails too, fall back
     # on plain old print statements.
     if (!$template->process("global/code-error.html.tmpl", $vars)) {
-        my $maintainer = Bugzilla::Config::Param('maintainer');
-        my $error = Bugzilla::Util::html_quote($vars->{'template_error_msg'});
-        my $error2 = Bugzilla::Util::html_quote($template->error());
+        my $maintainer = Bugzilla->params->{'maintainer'};
+        my $error = html_quote($vars->{'template_error_msg'});
+        my $error2 = html_quote($template->error());
         print <<END;
         <tt>
           <p>
@@ -160,7 +187,7 @@ Bugzilla::Error - Error handling utilities for Bugzilla
 
 Various places throughout the Bugzilla codebase need to report errors to the
 user. The C<Throw*Error> family of functions allow this to be done in a
-generic and localisable manner.
+generic and localizable manner.
 
 These functions automatically unlock the database tables, if there were any
 locked. They will also roll back the transaction, if it is supported by
