@@ -34,22 +34,28 @@ package Bugzilla::Search;
 use base qw(Exporter);
 @Bugzilla::Search::EXPORT = qw(IsValidQueryType);
 
-use Bugzilla::Config;
 use Bugzilla::Error;
 use Bugzilla::Util;
 use Bugzilla::Constants;
 use Bugzilla::Group;
 use Bugzilla::User;
 use Bugzilla::Field;
+use Bugzilla::Bug;
+use Bugzilla::Keyword;
 
 use Date::Format;
 use Date::Parse;
 
+# How much we should add to the relevance, for each word that matches
+# in bugs.short_desc, during fulltext search. This is high because
+# we want summary matches to be *very* relevant, by default.
+use constant SUMMARY_RELEVANCE_FACTOR => 7;
+
 # Some fields are not sorted on themselves, but on other fields. 
 # We need to have a list of these fields and what they map to.
 # Each field points to an array that contains the fields mapped 
 # to, in order.
-our %specialorder = (
+use constant SPECIAL_ORDER => {
     'bugs.target_milestone' => [ 'ms_order.sortkey','ms_order.value' ],
     'bugs.bug_status' => [ 'bug_status.sortkey','bug_status.value' ],
     'bugs.rep_platform' => [ 'rep_platform.sortkey','rep_platform.value' ],
@@ -57,12 +63,12 @@ our %specialorder = (
     'bugs.op_sys' => [ 'op_sys.sortkey','op_sys.value' ],
     'bugs.resolution' => [ 'resolution.sortkey', 'resolution.value' ],
     'bugs.bug_severity' => [ 'bug_severity.sortkey','bug_severity.value' ]
-);
+};
 
 # When we add certain fields to the ORDER BY, we need to then add a
 # table join to the FROM statement. This hash maps input fields to 
-# the join statements that ned to be added.
-our %specialorderjoin = (
+# the join statements that need to be added.
+use constant SPECIAL_ORDER_JOIN => {
     'bugs.target_milestone' => 'LEFT JOIN milestones AS ms_order ON ms_order.value = bugs.target_milestone AND ms_order.product_id = bugs.product_id',
     'bugs.bug_status' => 'LEFT JOIN bug_status ON bug_status.value = bugs.bug_status',
     'bugs.rep_platform' => 'LEFT JOIN rep_platform ON rep_platform.value = bugs.rep_platform',
@@ -70,7 +76,7 @@ our %specialorderjoin = (
     'bugs.op_sys' => 'LEFT JOIN op_sys ON op_sys.value = bugs.op_sys',
     'bugs.resolution' => 'LEFT JOIN resolution ON resolution.value = bugs.resolution',
     'bugs.bug_severity' => 'LEFT JOIN bug_severity ON bug_severity.value = bugs.bug_severity'
-);
+};
 
 # Create a new Search
 # Note that the param argument may be modified by Bugzilla::Search
@@ -111,10 +117,20 @@ sub init {
     my @andlist;
     my %chartfields;
 
+    my %special_order      = %{SPECIAL_ORDER()};
+    my %special_order_join = %{SPECIAL_ORDER_JOIN()};
+
+    my @select_fields = Bugzilla->get_fields({ type => FIELD_TYPE_SINGLE_SELECT,
+                                               obsolete => 0 });
+    foreach my $field (@select_fields) {
+        my $name = $field->name;
+        $special_order{"bugs.$name"} = [ "$name.sortkey", "$name.value" ],
+        $special_order_join{"bugs.$name"} =
+            "LEFT JOIN $name ON $name.value = bugs.$name";
+    }
+
     my $dbh = Bugzilla->dbh;
 
-    &::GetVersionTable();
-    
     # First, deal with all the old hard-coded non-chart-based poop.
     if (grep(/map_assigned_to/, @$fieldsref)) {
         push @supptables, "INNER JOIN profiles AS map_assigned_to " .
@@ -150,7 +166,7 @@ sub init {
     }
     
     if (grep($_ =~/AS (actual_time|percentage_complete)$/, @$fieldsref)) {
-        push(@supptables, "INNER JOIN longdescs AS ldtime " .
+        push(@supptables, "LEFT JOIN longdescs AS ldtime " .
                           "ON ldtime.bug_id = bugs.bug_id");
     }
 
@@ -180,25 +196,26 @@ sub init {
     # into their equivalent lists of open and closed statuses.
     if ($params->param('bug_status')) {
         my @bug_statuses = $params->param('bug_status');
-        if (scalar(@bug_statuses) == scalar(@::legal_bug_status) 
+        my @legal_statuses = @{get_legal_field_values('bug_status')};
+        if (scalar(@bug_statuses) == scalar(@legal_statuses)
             || $bug_statuses[0] eq "__all__")
         {
             $params->delete('bug_status');
         }
         elsif ($bug_statuses[0] eq '__open__') {
-            $params->param('bug_status', map(&::IsOpenedState($_) ? $_ : undef, 
-                                             @::legal_bug_status));
+            $params->param('bug_status', grep(is_open_state($_), 
+                                              @legal_statuses));
         }
         elsif ($bug_statuses[0] eq "__closed__") {
-            $params->param('bug_status', map(&::IsOpenedState($_) ? undef : $_, 
-                                             @::legal_bug_status));
+            $params->param('bug_status', grep(!is_open_state($_), 
+                                              @legal_statuses));
         }
     }
     
     if ($params->param('resolution')) {
         my @resolutions = $params->param('resolution');
-        
-        if (scalar(@resolutions) == scalar(@::legal_resolution)) {
+        my $legal_resolutions = get_legal_field_values('resolution');
+        if (scalar(@resolutions) == scalar(@$legal_resolutions)) {
             $params->delete('resolution');
         }
     }
@@ -208,6 +225,9 @@ sub init {
                         "assigned_to", "reporter", "component", "classification",
                         "target_milestone", "bug_group");
 
+    # Include custom select fields.
+    push(@legal_fields, map { $_->name } @select_fields);
+
     foreach my $field ($params->param()) {
         if (lsearch(\@legal_fields, $field) != -1) {
             push(@specialchart, [$field, "anyexact",
@@ -237,7 +257,7 @@ sub init {
             foreach my $name (split(',', $email)) {
                 $name = trim($name);
                 if ($name) {
-                    &::DBNameToIdAndCheck($name);
+                    login_to_id($name, THROW_ERROR);
                 }
             }
         }
@@ -293,15 +313,17 @@ sub init {
     }
 
     if ($chfieldfrom ne '' || $chfieldto ne '') {
-        my $sql_chfrom = $chfieldfrom ? &::SqlQuote(SqlifyDate($chfieldfrom)):'';
-        my $sql_chto   = $chfieldto   ? &::SqlQuote(SqlifyDate($chfieldto))  :'';
-        my $sql_chvalue = $chvalue ne '' ? &::SqlQuote($chvalue) : '';
+        my $sql_chfrom = $chfieldfrom ? $dbh->quote(SqlifyDate($chfieldfrom)):'';
+        my $sql_chto   = $chfieldto   ? $dbh->quote(SqlifyDate($chfieldto))  :'';
+        my $sql_chvalue = $chvalue ne '' ? $dbh->quote($chvalue) : '';
+        trick_taint($sql_chvalue);
         if(!@chfield) {
             push(@wherepart, "bugs.delta_ts >= $sql_chfrom") if ($sql_chfrom);
             push(@wherepart, "bugs.delta_ts <= $sql_chto") if ($sql_chto);
         } else {
             my $bug_creation_clause;
             my @list;
+            my @actlist;
             foreach my $f (@chfield) {
                 if ($f eq "[Bug creation]") {
                     # Treat [Bug creation] differently because we need to look
@@ -311,14 +333,15 @@ sub init {
                     push(@l, "bugs.creation_ts <= $sql_chto") if($sql_chto);
                     $bug_creation_clause = "(" . join(' AND ', @l) . ")";
                 } else {
-                    push(@list, "\nactcheck.fieldid = " . get_field_id($f));
+                    push(@actlist, get_field_id($f));
                 }
             }
 
-            # @list won't have any elements if the only field being searched
+            # @actlist won't have any elements if the only field being searched
             # is [Bug creation] (in which case we don't need bugs_activity).
-            if(@list) {
-                my $extra = "";
+            if(@actlist) {
+                my $extra = " actcheck.bug_id = bugs.bug_id";
+                push(@list, "(actcheck.bug_when IS NOT NULL)");
                 if($sql_chfrom) {
                     $extra .= " AND actcheck.bug_when >= $sql_chfrom";
                 }
@@ -328,8 +351,9 @@ sub init {
                 if($sql_chvalue) {
                     $extra .= " AND actcheck.added = $sql_chvalue";
                 }
-                push(@supptables, "INNER JOIN bugs_activity AS actcheck " .
-                                   "ON actcheck.bug_id = bugs.bug_id $extra");
+                push(@supptables, "LEFT JOIN bugs_activity AS actcheck " .
+                                  "ON $extra AND actcheck.fieldid IN (" .
+                                  join(",", @actlist) . ")");
             }
 
             # Now that we're done using @list to determine if there are any
@@ -344,7 +368,7 @@ sub init {
 
     my $sql_deadlinefrom;
     my $sql_deadlineto;
-    if (Bugzilla->user->in_group(Param('timetrackinggroup'))){
+    if (Bugzilla->user->in_group(Bugzilla->params->{'timetrackinggroup'})){
       my $deadlinefrom;
       my $deadlineto;
             
@@ -353,7 +377,8 @@ sub init {
         validate_date($deadlinefrom)
           || ThrowUserError('illegal_date', {date => $deadlinefrom,
                                              format => 'YYYY-MM-DD'});
-        $sql_deadlinefrom = &::SqlQuote($deadlinefrom);
+        $sql_deadlinefrom = $dbh->quote($deadlinefrom);
+        trick_taint($sql_deadlinefrom);
         push(@wherepart, "bugs.deadline >= $sql_deadlinefrom");
       }
       
@@ -362,7 +387,8 @@ sub init {
         validate_date($deadlineto)
           || ThrowUserError('illegal_date', {date => $deadlineto,
                                              format => 'YYYY-MM-DD'});
-        $sql_deadlineto = &::SqlQuote($deadlineto);
+        $sql_deadlineto = $dbh->quote($deadlineto);
+        trick_taint($sql_deadlineto);
         push(@wherepart, "bugs.deadline <= $sql_deadlineto");
       }
     }  
@@ -373,7 +399,8 @@ sub init {
             my $s = trim($params->param($f));
             if ($s ne "") {
                 my $n = $f;
-                my $q = &::SqlQuote($s);
+                my $q = $dbh->quote($s);
+                trick_taint($q);
                 my $type = $params->param($f . "_type");
                 push(@specialchart, [$f, $type, $s]);
             }
@@ -381,15 +408,7 @@ sub init {
     }
 
     if (defined $params->param('content')) {
-        # Append a new chart implementing content quicksearch
-        my $chart;
-        for ($chart = 0 ; $params->param("field$chart-0-0") ; $chart++) {};
-        $params->param("field$chart-0-0", 'content');
-        $params->param("type$chart-0-0", 'matches');
-        $params->param("value$chart-0-0", $params->param('content'));
-        $params->param("field$chart-0-1", 'short_desc');
-        $params->param("type$chart-0-1", 'allwords');
-        $params->param("value$chart-0-1", $params->param('content'));
+        push(@specialchart, ['content', 'matches', $params->param('content')]);
     }
 
     my $chartid;
@@ -405,7 +424,7 @@ sub init {
     my %funcsbykey;
     my @funcdefs =
         (
-         "^(?:assigned_to|reporter|qa_contact),(?:notequals|equals|anyexact),%group\\.(\\w+)%" => sub {
+         "^(?:assigned_to|reporter|qa_contact),(?:notequals|equals|anyexact),%group\\.([^%]+)%" => sub {
              my $group = $1;
              my $groupid = Bugzilla::Group::ValidateGroupName( $group, ($user));
              $groupid || ThrowUserError('invalid_group_name',{name => $group});
@@ -447,7 +466,7 @@ sub init {
              $f = "COALESCE(map_$f.login_name,'')";
          },
 
-         "^(?:cc),(?:notequals|equals|anyexact),%group\\.(\\w+)%" => sub {
+         "^(?:cc),(?:notequals|equals|anyexact),%group\\.([^%]+)%" => sub {
              my $group = $1;
              my $groupid = Bugzilla::Group::ValidateGroupName( $group, ($user));
              $groupid || ThrowUserError('invalid_group_name',{name => $group});
@@ -546,22 +565,22 @@ sub init {
 
          "^long_?desc,changedby" => sub {
              my $table = "longdescs_$chartid";
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON $table.bug_id = bugs.bug_id");
-             my $id = &::DBNameToIdAndCheck($v);
+             my $id = login_to_id($v, THROW_ERROR);
              $term = "$table.who = $id";
          },
          "^long_?desc,changedbefore" => sub {
              my $table = "longdescs_$chartid";
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON $table.bug_id = bugs.bug_id");
-             $term = "$table.bug_when < " . &::SqlQuote(SqlifyDate($v));
+             $term = "$table.bug_when < " . $dbh->quote(SqlifyDate($v));
          },
          "^long_?desc,changedafter" => sub {
              my $table = "longdescs_$chartid";
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON $table.bug_id = bugs.bug_id");
-             $term = "$table.bug_when > " . &::SqlQuote(SqlifyDate($v));
+             $term = "$table.bug_when > " . $dbh->quote(SqlifyDate($v));
          },
          "^content,matches" => sub {
              # "content" is an alias for columns containing text for which we
@@ -574,40 +593,38 @@ sub init {
              # Add the longdescs table to the query so we can search comments.
              my $table = "longdescs_$chartid";
              my $extra = "";
-             if (Param("insidergroup") 
-                 && !UserInGroup(Param("insidergroup")))
+             if (Bugzilla->params->{"insidergroup"} 
+                 && !Bugzilla->user->in_group(Bugzilla->params->{"insidergroup"}))
              {
                  $extra = "AND $table.isprivate < 1";
              }
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON bugs.bug_id = $table.bug_id $extra");
 
              # Create search terms to add to the SELECT and WHERE clauses.
              # $term1 searches comments.
-             # $term2 searches summaries, which contributes to the relevance
-             # ranking in SELECT but doesn't limit which bugs get retrieved.
              my $term1 = $dbh->sql_fulltext_search("${table}.thetext", $v);
-             my $term2 = $dbh->sql_fulltext_search("bugs.short_desc", $v);
+
+             # short_desc searching for the WHERE clause
+             my @words = _split_words_into_like('bugs.short_desc', $v);
+             my $term2_where = join(' OR ', @words);
+
+             # short_desc relevance
+             my $factor = SUMMARY_RELEVANCE_FACTOR;
+             my @s_words = map("CASE WHEN $_ THEN $factor ELSE 0 END", @words);
+             my $term2_select = join(' + ', @s_words);
 
              # The term to use in the WHERE clause.
-             $term = "$term1 > 0";
+             $term = "$term1 > 0 OR ($term2_where)";
 
              # In order to sort by relevance (in case the user requests it),
              # we SELECT the relevance value and give it an alias so we can
              # add it to the SORT BY clause when we build it in buglist.cgi.
              #
-             # Note: MySQL calculates relevance for each comment separately,
-             # so we need to do some additional calculations to get an overall
-             # relevance value, which we do by calculating the average (mean)
-             # comment relevance and then adding the summary relevance, if any.
-             # This weights summary relevance heavily, which makes sense
-             # since summaries are short and thus highly significant.
-             #
-             # Note: We should be calculating the average relevance of all
+             # Note: We should be calculating the relevance based on all
              # comments for a bug, not just matching comments, but that's hard
              # (see http://bugzilla.mozilla.org/show_bug.cgi?id=145588#c35).
-             my $select_term =
-               "(SUM($term1)/COUNT($term1) + $term2) AS relevance";
+             my $select_term = "(SUM($term1) + $term2_select) AS relevance";
 
              # add the column not used in aggregate function explicitly
              push(@groupby, 'bugs.short_desc');
@@ -626,9 +643,9 @@ sub init {
          "^content," => sub {
              ThrowUserError("search_content_without_matches");
          },
-         "^deadline,(?:lessthan|greaterthan|equals|notequals),(-|\\+)?(\\d+)([dDwWmMyY])\$" => sub {
+         "^(?:deadline|creation_ts|delta_ts),(?:lessthan|greaterthan|equals|notequals),(?:-|\\+)?(?:\\d+)(?:[dDwWmMyY])\$" => sub {
              $v = SqlifyDate($v);
-             $q = &::SqlQuote($v);
+             $q = $dbh->quote($v);
         },
          "^commenter,(?:equals|anyexact),(%\\w+%)" => sub {
              my $match = pronoun($1, $user);
@@ -639,7 +656,9 @@ sub init {
              }
              my $table = "longdescs_$chartseq";
              my $extra = "";
-             if (Param("insidergroup") && !UserInGroup(Param("insidergroup"))) {
+             if (Bugzilla->params->{"insidergroup"} 
+                 && !Bugzilla->user->in_group(Bugzilla->params->{"insidergroup"})) 
+             {
                  $extra = "AND $table.isprivate < 1";
              }
              push(@supptables, "LEFT JOIN longdescs AS $table " .
@@ -657,7 +676,9 @@ sub init {
              }
              my $table = "longdescs_$chartseq";
              my $extra = "";
-             if (Param("insidergroup") && !UserInGroup(Param("insidergroup"))) {
+             if (Bugzilla->params->{"insidergroup"} 
+                 && !Bugzilla->user->in_group(Bugzilla->params->{"insidergroup"})) 
+             {
                  $extra = "AND $table.isprivate < 1";
              }
              if ($list) {
@@ -678,38 +699,52 @@ sub init {
          "^long_?desc," => sub {
              my $table = "longdescs_$chartid";
              my $extra = "";
-             if (Param("insidergroup") && !UserInGroup(Param("insidergroup"))) {
+             if (Bugzilla->params->{"insidergroup"} 
+                 && !Bugzilla->user->in_group(Bugzilla->params->{"insidergroup"})) 
+             {
                  $extra = "AND $table.isprivate < 1";
              }
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON $table.bug_id = bugs.bug_id $extra");
              $f = "$table.thetext";
          },
+         "^longdescs\.isprivate," => sub {
+             my $table = "longdescs_$chartid";
+             my $extra = "";
+             if (Bugzilla->params->{"insidergroup"}
+                 && !Bugzilla->user->in_group(Bugzilla->params->{"insidergroup"}))
+             {
+                 $extra = "AND $table.isprivate < 1";
+             }
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
+                               "ON $table.bug_id = bugs.bug_id $extra");
+             $f = "$table.isprivate";
+         },
          "^work_time,changedby" => sub {
              my $table = "longdescs_$chartid";
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON $table.bug_id = bugs.bug_id");
-             my $id = &::DBNameToIdAndCheck($v);
+             my $id = login_to_id($v, THROW_ERROR);
              $term = "(($table.who = $id";
              $term .= ") AND ($table.work_time <> 0))";
          },
          "^work_time,changedbefore" => sub {
              my $table = "longdescs_$chartid";
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON $table.bug_id = bugs.bug_id");
-             $term = "(($table.bug_when < " . &::SqlQuote(SqlifyDate($v));
+             $term = "(($table.bug_when < " . $dbh->quote(SqlifyDate($v));
              $term .= ") AND ($table.work_time <> 0))";
          },
          "^work_time,changedafter" => sub {
              my $table = "longdescs_$chartid";
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON $table.bug_id = bugs.bug_id");
-             $term = "(($table.bug_when > " . &::SqlQuote(SqlifyDate($v));
+             $term = "(($table.bug_when > " . $dbh->quote(SqlifyDate($v));
              $term .= ") AND ($table.work_time <> 0))";
          },
          "^work_time," => sub {
              my $table = "longdescs_$chartid";
-             push(@supptables, "INNER JOIN longdescs AS $table " .
+             push(@supptables, "LEFT JOIN longdescs AS $table " .
                                "ON $table.bug_id = bugs.bug_id");
              $f = "$table.work_time";
          },
@@ -741,7 +776,7 @@ sub init {
                  if(lsearch(\@fields, "bugs.remaining_time") == -1) {
                      push(@fields, "bugs.remaining_time");                  
                  }
-                 push(@supptables, "INNER JOIN longdescs AS $table " .
+                 push(@supptables, "LEFT JOIN longdescs AS $table " .
                                    "ON $table.bug_id = bugs.bug_id");
                  my $expression = "(100 * ((SUM($table.work_time) *
                                              COUNT(DISTINCT $table.bug_when) /
@@ -750,12 +785,14 @@ sub init {
                                               COUNT(DISTINCT $table.bug_when) /
                                               COUNT(bugs.bug_id)) +
                                              bugs.remaining_time)))";
+                 $q = $dbh->quote($v);
+                 trick_taint($q);
                  if ($t eq "regexp") {
-                     push(@having, $dbh->sql_regexp($expression, &::SqlQuote($v)));
+                     push(@having, $dbh->sql_regexp($expression, $q));
                  } elsif ($t eq "notregexp") {
-                     push(@having, $dbh->sql_not_regexp($expression, &::SqlQuote($v)));
+                     push(@having, $dbh->sql_not_regexp($expression, $q));
                  } else {
-                     push(@having, "$expression $oper " . &::SqlQuote($v));
+                     push(@having, "$expression $oper " . $q);
                  }
                  push(@groupby, "bugs.remaining_time");
              }
@@ -783,7 +820,9 @@ sub init {
              my $atable = "attachments_$chartid";
              my $dtable = "attachdata_$chartid";
              my $extra = "";
-             if (Param("insidergroup") && !UserInGroup(Param("insidergroup"))) {
+             if (Bugzilla->params->{"insidergroup"} 
+                 && !Bugzilla->user->in_group(Bugzilla->params->{"insidergroup"})) 
+             {
                  $extra = "AND $atable.isprivate = 0";
              }
              push(@supptables, "INNER JOIN attachments AS $atable " .
@@ -792,10 +831,26 @@ sub init {
                                "ON $dtable.id = $atable.attach_id");
              $f = "$dtable.thedata";
          },
+         "^attachments\.submitter," => sub {
+             my $atable = "map_attachment_submitter_$chartid";
+             my $extra = "";
+             if (Bugzilla->params->{"insidergroup"}
+                 && !Bugzilla->user->in_group(Bugzilla->params->{"insidergroup"}))
+             {
+                 $extra = "AND $atable.isprivate = 0";
+             }
+             push(@supptables, "INNER JOIN attachments AS $atable " .
+                               "ON bugs.bug_id = $atable.bug_id $extra");
+             push(@supptables, "LEFT JOIN profiles AS attachers_$chartid " .
+                               "ON $atable.submitter_id = attachers_$chartid.userid");
+             $f = "attachers_$chartid.login_name";
+         },
          "^attachments\..*," => sub {
              my $table = "attachments_$chartid";
              my $extra = "";
-             if (Param("insidergroup") && !UserInGroup(Param("insidergroup"))) {
+             if (Bugzilla->params->{"insidergroup"} 
+                 && !Bugzilla->user->in_group(Bugzilla->params->{"insidergroup"})) 
+             {
                  $extra = "AND $table.isprivate = 0";
              }
              push(@supptables, "INNER JOIN attachments AS $table " .
@@ -803,18 +858,18 @@ sub init {
              $f =~ m/^attachments\.(.*)$/;
              my $field = $1;
              if ($t eq "changedby") {
-                 $v = &::DBNameToIdAndCheck($v);
-                 $q = &::SqlQuote($v);
+                 $v = login_to_id($v, THROW_ERROR);
+                 $q = $dbh->quote($v);
                  $field = "submitter_id";
                  $t = "equals";
              } elsif ($t eq "changedbefore") {
                  $v = SqlifyDate($v);
-                 $q = &::SqlQuote($v);
+                 $q = $dbh->quote($v);
                  $field = "creation_ts";
                  $t = "lessthan";
              } elsif ($t eq "changedafter") {
                  $v = SqlifyDate($v);
-                 $q = &::SqlQuote($v);
+                 $q = $dbh->quote($v);
                  $field = "creation_ts";
                  $t = "greaterthan";
              }
@@ -841,8 +896,7 @@ sub init {
              # negative conditions (f.e. "flag isn't review+").
              my $flags = "flags_$chartid";
              push(@supptables, "LEFT JOIN flags AS $flags " . 
-                               "ON bugs.bug_id = $flags.bug_id " .
-                               "AND $flags.is_active = 1");
+                               "ON bugs.bug_id = $flags.bug_id ");
              my $flagtypes = "flagtypes_$chartid";
              push(@supptables, "LEFT JOIN flagtypes AS $flagtypes " . 
                                "ON $flags.type_id = $flagtypes.id");
@@ -872,8 +926,7 @@ sub init {
          "^requestees.login_name," => sub {
              my $flags = "flags_$chartid";
              push(@supptables, "LEFT JOIN flags AS $flags " .
-                               "ON bugs.bug_id = $flags.bug_id " .
-                               "AND $flags.is_active = 1");
+                               "ON bugs.bug_id = $flags.bug_id ");
              push(@supptables, "LEFT JOIN profiles AS requestees_$chartid " .
                                "ON $flags.requestee_id = requestees_$chartid.userid");
              $f = "requestees_$chartid.login_name";
@@ -881,8 +934,7 @@ sub init {
          "^setters.login_name," => sub {
              my $flags = "flags_$chartid";
              push(@supptables, "LEFT JOIN flags AS $flags " .
-                               "ON bugs.bug_id = $flags.bug_id " .
-                               "AND $flags.is_active = 1");
+                               "ON bugs.bug_id = $flags.bug_id ");
              push(@supptables, "LEFT JOIN profiles AS setters_$chartid " .
                                "ON $flags.setter_id = setters_$chartid.userid");
              $f = "setters_$chartid.login_name";
@@ -925,16 +977,15 @@ sub init {
          },
 
          "^keywords,(?!changed)" => sub {
-             &::GetVersionTable();
              my @list;
              my $table = "keywords_$chartid";
              foreach my $value (split(/[\s,]+/, $v)) {
                  if ($value eq '') {
                      next;
                  }
-                 my $id = &::GetKeywordIdFromName($value);
-                 if ($id) {
-                     push(@list, "$table.keywordid = $id");
+                 my $keyword = new Bugzilla::Keyword({name => $value});
+                 if ($keyword) {
+                     push(@list, "$table.keywordid = " . $keyword->id);
                  }
                  else {
                      ThrowUserError("unknown_keyword",
@@ -1061,10 +1112,12 @@ sub init {
          ",anyexact" => sub {
              my @list;
              foreach my $w (split(/,/, $v)) {
-                 if ($w eq "---" && $f !~ /milestone/) {
+                 if ($w eq "---" && $f =~ /resolution/) {
                      $w = "";
                  }
-                 push(@list, &::SqlQuote($w));
+                 $q = $dbh->quote($w);
+                 trick_taint($q);
+                 push(@list, $q);
              }
              if (@list) {
                  $term = "$ff IN (" . join (',', @list) . ")";
@@ -1105,7 +1158,7 @@ sub init {
                                "ON $table.bug_id = bugs.bug_id " .
                                "AND $table.fieldid = $fieldid " .
                                "AND $table.bug_when $operator " . 
-                               &::SqlQuote(SqlifyDate($v)) );
+                               $dbh->quote(SqlifyDate($v)) );
              $term = "($table.bug_when IS NOT NULL)";
          },
          ",(changedfrom|changedto)" => sub {
@@ -1127,7 +1180,7 @@ sub init {
              if (!$fieldid) {
                  ThrowCodeError("invalid_field_name", {field => $f});
              }
-             my $id = &::DBNameToIdAndCheck($v);
+             my $id = login_to_id($v, THROW_ERROR);
              push(@supptables, "LEFT JOIN bugs_activity AS $table " .
                                "ON $table.bug_id = bugs.bug_id " .
                                "AND $table.fieldid = $fieldid " .
@@ -1167,7 +1220,8 @@ sub init {
             $params->param("value$chart-$row-$col", shift(@$ref));
             if ($debug) {
                 push(@debugdata, "$row-$col = " .
-                               $params->param("field$chart-$row-$col") . ' | ' .                               $params->param("type$chart-$row-$col") . ' | ' .
+                               $params->param("field$chart-$row-$col") . ' | ' .
+                               $params->param("type$chart-$row-$col") . ' | ' .
                                $params->param("value$chart-$row-$col") . ' *');
             }
             $col++;
@@ -1251,7 +1305,7 @@ sub init {
 #       e.g. bugs_activity.bug_id
 # $t  = type of query. e.g. "equal to", "changed after", case sensitive substr"
 # $v  = value - value the user typed in to the form
-# $q  = sanitized version of user input (SqlQuote($v))
+# $q  = sanitized version of user input trick_taint(($dbh->quote($v)))
 # @supptables = Tables and/or table aliases used in query
 # %suppseen   = A hash used to store all the tables in supptables to weed
 #               out duplicates.
@@ -1260,11 +1314,8 @@ sub init {
 # $suppstring = String which is pasted into query containing all table names
 
     # get a list of field names to verify the user-submitted chart fields against
-    &::SendSQL("SELECT name, fieldid FROM fielddefs");
-    while (&::MoreSQLData()) {
-        my ($name, $id) = &::FetchSQLData();
-        $chartfields{$name} = $id;
-    }
+    %chartfields = @{$dbh->selectcol_arrayref(
+        q{SELECT name, id FROM fielddefs}, { Columns=>[1,2] })};
 
     $row = 0;
     for ($chart=-1 ;
@@ -1297,7 +1348,8 @@ sub init {
                 # already know about it), or it was in %chartfields, so it is
                 # a valid field name, which means that it's ok.
                 trick_taint($f);
-                $q = &::SqlQuote($v);
+                $q = $dbh->quote($v);
+                trick_taint($q);
                 my $rhs = $v;
                 $rhs =~ tr/,//;
                 my $func;
@@ -1358,7 +1410,7 @@ sub init {
         if ($orderitem =~ /\s+AS\s+(.+)$/i) {
             $orderitem = $1;
         }
-        BuildOrderBy($orderitem, \@orderby);
+        BuildOrderBy(\%special_order, $orderitem, \@orderby);
     }
     # Now JOIN the correct tables in the FROM clause.
     # This is done separately from the above because it's
@@ -1366,8 +1418,8 @@ sub init {
     foreach my $orderitem (@inputorder) {
         # Grab the part without ASC or DESC.
         my @splitfield = split(/\s+/, $orderitem);
-        if ($specialorderjoin{$splitfield[0]}) {
-            push(@supptables, $specialorderjoin{$splitfield[0]});
+        if ($special_order_join{$splitfield[0]}) {
+            push(@supptables, $special_order_join{$splitfield[0]});
         }
     }
 
@@ -1375,21 +1427,20 @@ sub init {
     my $suppstring = "bugs";
     my @supplist = (" ");
     foreach my $str (@supptables) {
-        if (!$suppseen{$str}) {
-            if ($str =~ /^(LEFT|INNER|RIGHT)\s+JOIN/i) {
-                $str =~ /^(.*?)\s+ON\s+(.*)$/i;
-                my ($leftside, $rightside) = ($1, $2);
-                if ($suppseen{$leftside}) {
-                    $supplist[$suppseen{$leftside}] .= " AND ($rightside)";
-                } else {
-                    $suppseen{$leftside} = scalar @supplist;
-                    push @supplist, " $leftside ON ($rightside)";
-                }
+
+        if ($str =~ /^(LEFT|INNER|RIGHT)\s+JOIN/i) {
+            $str =~ /^(.*?)\s+ON\s+(.*)$/i;
+            my ($leftside, $rightside) = ($1, $2);
+            if (defined $suppseen{$leftside}) {
+                $supplist[$suppseen{$leftside}] .= " AND ($rightside)";
             } else {
-                # Do not accept implicit joins using comma operator
-                # as they are not DB agnostic
-                ThrowCodeError("comma_operator_deprecated");
+                $suppseen{$leftside} = scalar @supplist;
+                push @supplist, " $leftside ON ($rightside)";
             }
+        } else {
+            # Do not accept implicit joins using comma operator
+            # as they are not DB agnostic
+            ThrowCodeError("comma_operator_deprecated");
         }
     }
     $suppstring .= join('', @supplist);
@@ -1418,7 +1469,7 @@ sub init {
         $query .= "    OR (bugs.reporter_accessible = 1 AND bugs.reporter = $userid) " .
               "    OR (bugs.cclist_accessible = 1 AND cc.who IS NOT NULL) " .
               "    OR (bugs.assigned_to = $userid) ";
-        if (Param('useqacontact')) {
+        if (Bugzilla->params->{'useqacontact'}) {
             $query .= "OR (bugs.qa_contact = $userid) ";
         }
     }
@@ -1525,24 +1576,24 @@ sub ListIDsForEmail {
         }
         $list = join(',', @list);
     } elsif ($type eq 'substring') {
-        &::SendSQL("SELECT userid FROM profiles WHERE " .
-            $dbh->sql_position(lc(::SqlQuote($email)), "LOWER(login_name)") .
-            " > 0 " . $dbh->sql_limit(51));
-        while (&::MoreSQLData()) {
-            my ($id) = &::FetchSQLData();
-            push(@list, $id);
-        }
+        my $sql_email = $dbh->quote($email);
+        trick_taint($sql_email);
+        my $result = $dbh->selectcol_arrayref(
+                    q{SELECT userid FROM profiles WHERE } .
+                    $dbh->sql_position(lc($sql_email), q{LOWER(login_name)}) .
+                    q{ > 0 } . $dbh->sql_limit(51));
+        @list = @{$result};
         if (scalar(@list) < 50) {
             $list = join(',', @list);
         }
     } elsif ($type eq 'regexp') {
-        &::SendSQL("SELECT userid FROM profiles WHERE " .
-            $dbh->sql_regexp("login_name", ::SqlQuote($email)) .
-            " " . $dbh->sql_limit(51));
-        while (&::MoreSQLData()) {
-            my ($id) = &::FetchSQLData();
-            push(@list, $id);
-        }
+        my $sql_email = $dbh->quote($email);
+        trick_taint($sql_email);
+        my $result = $dbh->selectcol_arrayref(
+                        qq{SELECT userid FROM profiles WHERE } .
+                        $dbh->sql_regexp("login_name", $sql_email) .
+                        q{ } . $dbh->sql_limit(51));
+        @list = @{$result};
         if (scalar(@list) < 50) {
             $list = join(',', @list);
         }
@@ -1556,13 +1607,10 @@ sub build_subselect {
     my ($outer, $inner, $table, $cond) = @_;
     my $q = "SELECT $inner FROM $table WHERE $cond";
     #return "$outer IN ($q)";
-    &::SendSQL($q);
-    my @list;
-    while (&::MoreSQLData()) {
-        push (@list, &::FetchOneColumn());
-    }
-    return "1=2" unless @list; # Could use boolean type on dbs which support it
-    return "$outer IN (" . join(',', @list) . ")";
+    my $dbh = Bugzilla->dbh;
+    my $list = $dbh->selectcol_arrayref($q);
+    return "1=2" unless @$list; # Could use boolean type on dbs which support it
+    return "$outer IN (" . join(',', @$list) . ")";
 }
 
 sub GetByWordList {
@@ -1574,7 +1622,8 @@ sub GetByWordList {
         my $word = $w;
         if ($word ne "") {
             $word =~ tr/A-Z/a-z/;
-            $word = &::SqlQuote(quotemeta($word));
+            $word = $dbh->quote(quotemeta($word));
+            trick_taint($word);
             $word =~ s/^'//;
             $word =~ s/'$//;
             $word = '(^|[^a-z0-9])' . $word . '($|[^a-z0-9])';
@@ -1590,10 +1639,13 @@ sub GetByWordListSubstr {
     my ($field, $strs) = (@_);
     my @list;
     my $dbh = Bugzilla->dbh;
+    my $sql_word;
 
     foreach my $word (split(/[\s,]+/, $strs)) {
         if ($word ne "") {
-            push(@list, $dbh->sql_position(lc(::SqlQuote($word)),
+            $sql_word = $dbh->quote($word);
+            trick_taint($sql_word);
+            push(@list, $dbh->sql_position(lc($sql_word),
                                            "LOWER($field)") . " > 0");
         }
     }
@@ -1666,7 +1718,7 @@ sub IsValidQueryType
 # BuildOrderBy recursively, to let it know that we're "reversing" the 
 # order. That is, that we wanted "A DESC", not "A".
 sub BuildOrderBy {
-    my ($orderitem, $stringlist, $reverseorder) = (@_);
+    my ($special_order, $orderitem, $stringlist, $reverseorder) = (@_);
 
     my @twopart = split(/\s+/, $orderitem);
     my $orderfield = $twopart[0];
@@ -1684,15 +1736,31 @@ sub BuildOrderBy {
     }
 
     # Handle fields that have non-standard sort orders, from $specialorder.
-    if ($specialorder{$orderfield}) {
-        foreach my $subitem (@{$specialorder{$orderfield}}) {
+    if ($special_order->{$orderfield}) {
+        foreach my $subitem (@{$special_order->{$orderfield}}) {
             # DESC on a field with non-standard sort order means
             # "reverse the normal order for each field that we map to."
-            BuildOrderBy($subitem, $stringlist, $orderdirection =~ m/desc/i);
+            BuildOrderBy($special_order, $subitem, $stringlist,
+                         $orderdirection =~ m/desc/i);
         }
         return;
     }
 
     push(@$stringlist, trim($orderfield . ' ' . $orderdirection));
 }
+
+# This is a helper for fulltext search
+sub _split_words_into_like {
+    my ($field, $text) = @_;
+    my $dbh = Bugzilla->dbh;
+    # This code is very similar to Bugzilla::DB::sql_fulltext_search,
+    # so you can look there if you'd like an explanation of what it's
+    # doing.
+    my $lower_text = lc($text);
+    my @words = split(/\s+/, $lower_text);
+    @words = map($dbh->quote("%$_%"), @words);
+    map(trick_taint($_), @words);
+    @words = map($dbh->sql_istrcmp($field, $_, 'LIKE'), @words);
+    return @words;
+}
 1;
