@@ -23,17 +23,19 @@ package Bugzilla::Search::Quicksearch;
 # Make it harder for us to do dangerous things in Perl.
 use strict;
 
-use Bugzilla;
-use Bugzilla::Config;
 use Bugzilla::Error;
+use Bugzilla::Constants;
+use Bugzilla::Keyword;
+use Bugzilla::Bug;
+use Bugzilla::Field;
+use Bugzilla::Util;
 
 use base qw(Exporter);
 @Bugzilla::Search::Quicksearch::EXPORT = qw(quicksearch);
 
-my $cgi = Bugzilla->cgi;
-
 # Word renamings
-my %mappings = (# Status, Resolution, Platform, OS, Priority, Severity
+use constant MAPPINGS => {
+                # Status, Resolution, Platform, OS, Priority, Severity
                 "status" => "bug_status",
                 "resolution" => "resolution",  # no change
                 "platform" => "rep_platform",
@@ -83,26 +85,33 @@ my %mappings = (# Status, Resolution, Platform, OS, Priority, Severity
                 "attachmentdata" => "attach_data.thedata",
                 "attachdata" => "attach_data.thedata",
                 "attachmentmimetype" => "attachments.mimetype",
-                "attachmimetype" => "attachments.mimetype");
+                "attachmimetype" => "attachments.mimetype"
+};
 
 # We might want to put this into localconfig or somewhere
-my @platforms = ('pc', 'sun', 'macintosh', 'mac');
-my @productExceptions =   ('row'    # [Browser]
-                                    #   ^^^
-                          ,'new'    # [MailNews]
-                                    #      ^^^
-                          );
-my @componentExceptions = ('hang'   # [Bugzilla: Component/Keyword Changes]
-                                    #                               ^^^^
-                          );
+use constant PLATFORMS => ('pc', 'sun', 'macintosh', 'mac');
+use constant PRODUCT_EXCEPTIONS => (
+    'row',   # [Browser]
+             #   ^^^
+    'new',   # [MailNews]
+             #      ^^^
+);
+use constant COMPONENT_EXCEPTIONS => (
+    'hang'   # [Bugzilla: Component/Keyword Changes]
+             #                               ^^^^
+);
 
 # Quicksearch-wide globals for boolean charts.
-my $chart = 0;
-my $and = 0;
-my $or = 0;
+our ($chart, $and, $or);
 
 sub quicksearch {
     my ($searchstring) = (@_);
+    my $cgi = Bugzilla->cgi;
+    my $urlbase = correct_urlbase();
+
+    $chart = 0;
+    $and   = 0;
+    $or    = 0;
 
     # Remove leading and trailing commas and whitespace.
     $searchstring =~ s/(^[\s,]+|[\s,]+$)//g;
@@ -116,8 +125,7 @@ sub quicksearch {
 
         if (index($searchstring, ',') < $[) {
             # Single bug number; shortcut to show_bug.cgi.
-            print $cgi->redirect(-uri => Param('urlbase') .
-                                         "show_bug.cgi?id=$searchstring");
+            print $cgi->redirect(-uri => "${urlbase}show_bug.cgi?id=$searchstring");
             exit;
         }
         else {
@@ -136,15 +144,14 @@ sub quicksearch {
                                                   WHERE alias = ?},
                                                undef,
                                                $1)) {
-                print $cgi->redirect(-uri => Param('urlbase') .
-                                             "show_bug.cgi?id=$1");
+                print $cgi->redirect(-uri => "${urlbase}show_bug.cgi?id=$1");
                 exit;
             }
         }
 
         # It's no alias either, so it's a more complex query.
-
-        &::GetVersionTable();
+        my $legal_statuses = get_legal_field_values('bug_status');
+        my $legal_resolutions = get_legal_field_values('resolution');
 
         # Globally translate " AND ", " OR ", " NOT " to space, pipe, dash.
         $searchstring =~ s/\s+AND\s+/ /g;
@@ -152,17 +159,19 @@ sub quicksearch {
         $searchstring =~ s/\s+NOT\s+/ -/g;
 
         my @words = splitString($searchstring);
-        my $searchComments = $#words < Param('quicksearch_comment_cutoff');
-        my @openStates = &::OpenStates();
+        my $searchComments = 
+            $#words < Bugzilla->params->{'quicksearch_comment_cutoff'};
+        my @openStates = BUG_STATE_OPEN;
         my @closedStates;
+        my @unknownFields;
         my (%states, %resolutions);
 
-        foreach (@::legal_bug_status) {
-            push(@closedStates, $_) unless &::IsOpenedState($_);
+        foreach (@$legal_statuses) {
+            push(@closedStates, $_) unless is_open_state($_);
         }
         foreach (@openStates) { $states{$_} = 1 }
         if ($words[0] eq 'ALL') {
-            foreach (@::legal_bug_status) { $states{$_} = 1 }
+            foreach (@$legal_statuses) { $states{$_} = 1 }
             shift @words;
         }
         elsif ($words[0] eq 'OPEN') {
@@ -174,7 +183,7 @@ sub quicksearch {
                               \%resolutions,
                               [split(/,/, substr($words[0], 1))],
                               \@closedStates,
-                              \@::legal_resolution)) {
+                              $legal_resolutions)) {
                 shift @words;
                 # Allowing additional resolutions means we need to keep
                 # the "no resolution" resolution.
@@ -190,8 +199,8 @@ sub quicksearch {
             if (matchPrefixes(\%states,
                               \%resolutions,
                               [split(/,/, $words[0])],
-                              \@::legal_bug_status,
-                              \@::legal_resolution)) {
+                              $legal_statuses,
+                              $legal_resolutions)) {
                 shift @words;
             }
             else {
@@ -258,16 +267,19 @@ sub quicksearch {
                 foreach my $or_operand (split(/\|/, $qsword)) {
                     if ($or_operand =~ /^votes:([0-9]+)$/) {
                         # votes:xx ("at least xx votes")
-                        addChart('votes', 'greaterthan', $1, $negate);
+                        addChart('votes', 'greaterthan', $1 - 1, $negate);
                     }
                     elsif ($or_operand =~ /^([^:]+):([^:]+)$/) {
                         # generic field1,field2,field3:value1,value2 notation
                         my @fields = split(/,/, $1);
                         my @values = split(/,/, $2);
                         foreach my $field (@fields) {
-                            # Be tolerant about unknown fields
-                            next unless defined($mappings{$field});
-                            $field = $mappings{$field};
+                            # Skip and record any unknown fields
+                            if (!defined(MAPPINGS->{$field})) {
+                                push(@unknownFields, $field);
+                                next;
+                            }
+                            $field = MAPPINGS->{$field};
                             foreach (@values) {
                                 addChart($field, 'substring', $_, $negate);
                             }
@@ -280,7 +292,7 @@ sub quicksearch {
                         # by comma, which is another legal boolean OR indicator.
                         foreach my $word (split(/,/, $or_operand)) {
                             # Platform
-                            if (grep({lc($word) eq $_} @platforms)) {
+                            if (grep({lc($word) eq $_} PLATFORMS)) {
                                 addChart('rep_platform', 'substring',
                                          $word, $negate);
                             }
@@ -291,7 +303,7 @@ sub quicksearch {
                             }
                             # Severity
                             elsif (grep({lc($word) eq substr($_, 0, 3)}
-                                        @::legal_severity)) {
+                                        @{get_legal_field_values('bug_severity')})) {
                                 addChart('bug_severity', 'substring',
                                          $word, $negate);
                             }
@@ -309,21 +321,21 @@ sub quicksearch {
                             else { # Default QuickSearch word
 
                                 if (!grep({lc($word) eq $_}
-                                          @productExceptions) &&
+                                          PRODUCT_EXCEPTIONS) &&
                                     length($word)>2
                                    ) {
                                     addChart('product', 'substring',
                                              $word, $negate);
                                 }
                                 if (!grep({lc($word) eq $_}
-                                          @componentExceptions) &&
+                                          COMPONENT_EXCEPTIONS) &&
                                     length($word)>2
                                    ) {
                                     addChart('component', 'substring',
                                              $word, $negate);
                                 }
                                 if (grep({lc($word) eq $_}
-                                         @::legal_keywords)) {
+                                         map($_->name, Bugzilla::Keyword->get_all))) {
                                     addChart('keywords', 'substring',
                                              $word, $negate);
                                     if (length($word)>2) {
@@ -368,8 +380,13 @@ sub quicksearch {
             $or = 0;
         } # foreach (@words)
 
-        # We've been very tolerant about invalid queries, so all that's left
-        # may be an empty query.
+        # Inform user about any unknown fields
+        if (scalar(@unknownFields)) {
+            ThrowUserError("quicksearch_unknown_field",
+                           { fields => \@unknownFields });
+        }
+
+        # Make sure we have some query terms left
         scalar($cgi->param())>0 || ThrowUserError("buglist_parameters_required");
     }
 
@@ -379,9 +396,8 @@ sub quicksearch {
 
     if ($cgi->param('load')) {
         # Param 'load' asks us to display the query in the advanced search form.
-        print $cgi->redirect(-uri => Param('urlbase') . "query.cgi?" .
-                                     "format=advanced&amp;" .
-                                     $modified_query_string);
+        print $cgi->redirect(-uri => "${urlbase}query.cgi?format=advanced&amp;"
+                             . $modified_query_string);
     }
 
     # Otherwise, pass the modified query string to the caller.
@@ -402,28 +418,22 @@ sub splitString {
     my @parts;
     my $i = 0;
 
-    # Escape backslashes
-    $string =~ s/\\/\\\//g;
-
     # Now split on quote sign; be tolerant about unclosed quotes
     @quoteparts = split(/"/, $string);
-    foreach (@quoteparts) {
-        # After every odd quote, escape whitespace
-        s/(\s)/\\$1/g if $i++ % 2;
+    foreach my $part (@quoteparts) {
+        # After every odd quote, quote special chars
+        $part = url_quote($part) if $i++ % 2;
     }
     # Join again
     $string = join('"', @quoteparts);
 
     # Now split on unescaped whitespace
-    @parts = split(/(?<!\\)\s+/, $string);
+    @parts = split(/\s+/, $string);
     foreach (@parts) {
-        # Restore whitespace
-        s/\\(\s)/$1/g;
-        # Restore backslashes
-        s/\\\//\\/g;
         # Remove quotes
         s/"//g;
     }
+                        
     return @parts;
 }
 
@@ -491,9 +501,10 @@ sub addChart {
 sub makeChart {
     my ($expr, $field, $type, $value) = @_;
 
+    my $cgi = Bugzilla->cgi;
     $cgi->param("field$expr", $field);
     $cgi->param("type$expr",  $type);
-    $cgi->param("value$expr", $value);
+    $cgi->param("value$expr", url_decode($value));
 }
 
 1;
