@@ -4,7 +4,7 @@
 #
 # Nick Barnes, Ravenbrook Limited, 2004-04-01.
 #
-# $Id: sendbugmail.pl,v 1.3 2005/02/24 23:42:48 mkanat%kerio.com Exp $
+# $Id: sendbugmail.pl,v 1.7 2006/07/03 21:42:47 mkanat%bugzilla.org Exp $
 # 
 # Bugzilla email script for Bugzilla 2.17.4 and later.  Invoke this to send
 # bugmail for a bug which has been changed directly in the database.
@@ -16,10 +16,13 @@
 
 use lib qw(.);
 
-require "globals.pl";
+use Bugzilla;
+use Bugzilla::Util;
 use Bugzilla::BugMail;
 use Bugzilla::User;
 
+my $dbh = Bugzilla->dbh;
+
 sub usage {
     print STDERR "Usage: $0 bug_id user_email\n";
     exit;
@@ -41,15 +44,16 @@ if (!($bugnum =~ /^(\d+)$/)) {
 
 detaint_natural($bugnum);
 
-SendSQL("SELECT bug_id FROM bugs WHERE bug_id = $bugnum");
+my ($id) = $dbh->selectrow_array("SELECT bug_id FROM bugs WHERE bug_id = ?", 
+                                 undef, $bugnum);
 
-if (!FetchOneColumn()) {
+if (!$id) {
   print STDERR "Bug number $bugnum does not exist.\n";
   usage();
 }
 
 # Validate the changer address.
-my $match = Param('emailregexp');
+my $match = Bugzilla->params->{'emailregexp'};
 if ($changer !~ /$match/) {
     print STDERR "Changer \"$changer\" doesn't match email regular expression.\n";
     usage();
