@@ -27,13 +27,13 @@
 use strict;
 
 use lib ".";
-require "globals.pl";
 
-use Bugzilla::Config qw(:DEFAULT $datadir);
+use Bugzilla;
 use Bugzilla::Constants;
 use Bugzilla::Search;
 use Bugzilla::User;
-use Bugzilla::BugMail;
+use Bugzilla::Mailer;
+use Bugzilla::Util;
 
 # create some handles that we'll need
 my $template = Bugzilla->template;
@@ -50,7 +50,7 @@ my @seen_schedules = ();
 # allow the database to keep their SQL compiled.
 my $sth_run_queries =
     $dbh->prepare("SELECT " .
-                  "id, query_name, title, onemailperbug " .
+                  "query_name, title, onemailperbug " .
                   "FROM whine_queries " .
                   "WHERE eventid=? " .
                   "ORDER BY sortkey");
@@ -93,39 +93,17 @@ my $sth_schedules_by_event = $dbh->prepare(
 # After that, it looks over each user to see if they have schedules that need
 # running, then runs those and generates the email messages.
 
-# exit quietly if the system is shut down
-if (Param('shutdownhtml')) {
-    exit;
-}
-
-
-# Send whines from the address in the 'maintainer' Parameter so that all
+# Send whines from the address in the 'mailfrom' Parameter so that all
 # Bugzilla-originated mail appears to come from a single address.
-my $fromaddress = Param('maintainer');
+my $fromaddress = Bugzilla->params->{'mailfrom'};
 
-if ($fromaddress !~ Param('emailregexp')) {
-    die "Cannot run.  " .
-        "The maintainer email address has not been properly set!\n";
-}
-
-# Check the nomail file for users who should not receive mail
-my %nomail;
-if (open(NOMAIL, '<', "$datadir/nomail")) {
-    while (<NOMAIL>) {
-        $nomail{trim($_)} = 1;
-    }
-}
-
-# get the current date and time from the database
-$sth = $dbh->prepare('SELECT ' . $dbh->sql_date_format('NOW()', '%y,%m,%d,%a,%H,%i'));
-$sth->execute;
-my ($now_year, $now_month, $now_day, $now_weekdayname, $now_hour, $now_minute) =
-        split(',', $sth->fetchrow_array);
-$sth->finish;
-
-# As DBs have different days numbering, use day name and convert it
-# to the range 0-6
-my $now_weekday = index("SunMonTueWedThuFriSat", $now_weekdayname) / 3;
+# get the current date and time
+my ($now_sec, $now_minute, $now_hour, $now_day, $now_month, $now_year, 
+    $now_weekday) = localtime;
+# Convert year to two digits
+$now_year = sprintf("%02d", $now_year % 100);
+# Convert the month to January being "1" instead of January being "0".
+$now_month++;
 
 my @daysinmonth = qw(0 31 28 31 30 31 30 31 31 30 31 30 31);
 # Alter February in case of a leap year.  This simple way to do it only
@@ -170,20 +148,21 @@ while (my ($schedule_id, $day, $time) = $sched_h->fetchrow_array) {
         # A time greater than now means it still has to run today
         elsif ($time >= $now_hour) {
             # set it to today + number of hours
-            $sth = $dbh->prepare( "UPDATE whine_schedules " .
-                   "SET run_next = CURRENT_DATE() + " .
-                   $dbh->sql_interval('? HOUR') .
-                   " WHERE id = ?");
+            $sth = $dbh->prepare("UPDATE whine_schedules " .
+                                 "SET run_next = CURRENT_DATE + " .
+                                 $dbh->sql_interval('?', 'HOUR') .
+                                 " WHERE id = ?");
             $sth->execute($time, $schedule_id);
         }
         # the target time is less than the current time
         else { # set it for the next applicable day
-            my $nextdate = &get_next_date($day);
-            $sth = $dbh->prepare( "UPDATE whine_schedules " .
-                   "SET run_next = ? + " .
-                   $dbh->sql_interval('? HOUR') .
-                   " WHERE id = ?");
-            $sth->execute($nextdate, $time, $schedule_id);
+            $day = &get_next_date($day);
+            $sth = $dbh->prepare("UPDATE whine_schedules " .
+                                 "SET run_next = (CURRENT_DATE + " .
+                                 $dbh->sql_interval('?', 'DAY') . ") + " .
+                                 $dbh->sql_interval('?', 'HOUR') .
+                                 " WHERE id = ?");
+            $sth->execute($day, $time, $schedule_id);
         }
 
     }
@@ -195,10 +174,11 @@ while (my ($schedule_id, $day, $time) = $sched_h->fetchrow_array) {
         # midnight
         my $target_time = ($time =~ /^\d+$/) ? $time : 0;
 
-        $sth = $dbh->prepare( "UPDATE whine_schedules " .
-               "SET run_next = ? + " .
-               $dbh->sql_interval('? HOUR') .
-               " WHERE id = ?");
+        $sth = $dbh->prepare("UPDATE whine_schedules " .
+                             "SET run_next = (CURRENT_DATE + " .
+                             $dbh->sql_interval('?', 'DAY') . ") + " .
+                             $dbh->sql_interval('?', 'HOUR') .
+                             " WHERE id = ?");
         $sth->execute($target_date, $target_time, $schedule_id);
     }
 }
@@ -228,10 +208,10 @@ sub get_next_event {
 
         $dbh->bz_lock_tables('whine_schedules WRITE',
                              'whine_events READ',
-                             'profiles WRITE',
+                             'profiles READ',
                              'groups READ',
                              'group_group_map READ',
-                             'user_group_map WRITE');
+                             'user_group_map READ');
 
         # Get the event ID for the first pending schedule
         $sth_next_scheduled_event->execute;
@@ -240,8 +220,7 @@ sub get_next_event {
         return undef unless $fetched;
         my ($eventid, $owner_id, $subject, $body) = @{$fetched};
 
-        my $owner = Bugzilla::User->new($owner_id,
-                                        DERIVE_GROUPS_TABLES_ALREADY_LOCKED);
+        my $owner = Bugzilla::User->new($owner_id);
 
         my $whineatothers = $owner->in_group('bz_canusewhineatothers');
 
@@ -263,7 +242,7 @@ sub get_next_event {
                             $user_objects{$mailto} = $owner;
                         }
                         elsif ($whineatothers) {
-                            $user_objects{$mailto} = Bugzilla::User->new($mailto,DERIVE_GROUPS_TABLES_ALREADY_LOCKED);
+                            $user_objects{$mailto} = Bugzilla::User->new($mailto);
                         }
                     }
                 }
@@ -275,14 +254,17 @@ sub get_next_event {
                     my $group_id = Bugzilla::Group::ValidateGroupName(
                         $groupname, $owner);
                     if ($group_id) {
+                        my $glist = join(',',
+                            @{Bugzilla::User->flatten_group_membership(
+                            $group_id)});
                         $sth = $dbh->prepare("SELECT user_id FROM " .
                                              "user_group_map " .
-                                             "WHERE group_id=?");
-                        $sth->execute($group_id);
+                                             "WHERE group_id IN ($glist)");
+                        $sth->execute();
                         for my $row (@{$sth->fetchall_arrayref}) {
                             if (not defined $user_objects{$row->[0]}) {
                                 $user_objects{$row->[0]} =
-                                    Bugzilla::User->new($row->[0],DERIVE_GROUPS_TABLES_ALREADY_LOCKED);
+                                    Bugzilla::User->new($row->[0]);
                             }
                         }
                     }
@@ -324,7 +306,7 @@ while (my $event = get_next_event) {
 
     # We loop for each target user because some of the queries will be using
     # subjective pronouns
-    Bugzilla->switch_to_shadow_db();
+    $dbh = Bugzilla->switch_to_shadow_db();
     for my $target (@{$event->{'mailto'}}) {
         my $args = {
             'subject'     => $event->{'subject'},
@@ -349,7 +331,7 @@ while (my $event = get_next_event) {
 
         mail($args);
     }
-    Bugzilla->switch_to_main_db();
+    $dbh = Bugzilla->switch_to_main_db();
 }
 
 ################################################################################
@@ -376,10 +358,11 @@ while (my $event = get_next_event) {
 #
 sub mail {
     my $args = shift;
+    my $addressee = $args->{recipient};
+    # Don't send mail to someone whose bugmail notification is disabled.
+    return if $addressee->email_disabled;
 
-    # Don't send mail to someone on the nomail list.
-    return if $nomail{$args->{'recipient'}->{'login'}};
-
+    my $template = Bugzilla->template_inner($addressee->settings->{'lang'}->{'value'});
     my $msg = ''; # it's a temporary variable to hold the template output
     $args->{'alternatives'} ||= [];
 
@@ -405,12 +388,13 @@ sub mail {
 
     # now produce a ready-to-mail mime-encoded message
 
-    $args->{'boundary'} = "-----=====-----" . $$ . "--" . time() . "-----";
+    $args->{'boundary'} = "----------" . $$ . "--" . time() . "-----";
 
     $template->process("whine/multipart-mime.txt.tmpl", $args, \$msg)
         or die($template->error());
 
-    Bugzilla::BugMail::MessageToMTA($msg);
+    Bugzilla->template_inner("");
+    MessageToMTA($msg);
 
     delete $args->{'boundary'};
     delete $args->{'alternatives'};
@@ -426,18 +410,19 @@ sub run_queries {
     my $return_queries = [];
 
     $sth_run_queries->execute($args->{'eventid'});
-    my $queries = {};
+    my @queries = ();
     for (@{$sth_run_queries->fetchall_arrayref}) {
-        $queries->{$_->[0]} = {
-            'name'          => $_->[1],
-            'title'         => $_->[2],
-            'onemailperbug' => $_->[3],
-            'bugs'          => [],
-        };
+        push(@queries,
+            {
+              'name'          => $_->[0],
+              'title'         => $_->[1],
+              'onemailperbug' => $_->[2],
+              'bugs'          => [],
+            }
+        );
     }
 
-    for my $query_id (keys %{$queries}) {
-        my $thisquery = $queries->{$query_id};
+    foreach my $thisquery (@queries) {
         next unless $thisquery->{'name'};   # named query is blank
 
         my $savedquery = get_query($thisquery->{'name'}, $args->{'author'});
@@ -565,6 +550,10 @@ sub reset_timer {
     $sth->execute($schedule_id);
     my ($run_day, $run_time) = $sth->fetchrow_array;
 
+    # It may happen that the run_time field is NULL or blank due to
+    # a bug in editwhines.cgi when this field was initially 0.
+    $run_time ||= 0;
+
     my $run_today = 0;
     my $minute_offset = 0;
 
@@ -596,23 +585,25 @@ sub reset_timer {
 
         my $nextdate = &get_next_date($run_day);
 
-        $sth = $dbh->prepare( "UPDATE whine_schedules " .
-                              "SET run_next = ? + " .
-                              $dbh->sql_interval('? HOUR') .
-                              " WHERE id = ?");
+        $sth = $dbh->prepare("UPDATE whine_schedules " .
+                             "SET run_next = (CURRENT_DATE + " .
+                             $dbh->sql_interval('?', 'DAY') . ") + " .
+                             $dbh->sql_interval('?', 'HOUR') .
+                             " WHERE id = ?");
         $sth->execute($nextdate, $target_time, $schedule_id);
         return;
     }
 
-    # Scheduling is done in terms of whole minutes, so we use subtraction to
-    # drop the seconds from the time.
     if ($minute_offset > 0) {
+        # Scheduling is done in terms of whole minutes.
+        my $next_run = $dbh->selectrow_array('SELECT NOW() + ' .
+                                             $dbh->sql_interval('?', 'MINUTE'),
+                                             undef, $minute_offset);
+        $next_run = format_time($next_run, "%Y-%m-%d %R");
+
         $sth = $dbh->prepare("UPDATE whine_schedules " .
-                             "SET run_next = NOW() + " .
-                             $dbh->sql_interval('? MINUTE') . " - " .
-                             $dbh->sql_interval('SECOND(NOW()) SECOND') .
-                             " WHERE id = ?");
-        $sth->execute($minute_offset, $schedule_id);
+                             "SET run_next = ? WHERE id = ?");
+        $sth->execute($next_run, $schedule_id);
     } else {
         # The minute offset is zero or less, which is not supposed to happen.
         # complain to STDERR
@@ -636,7 +627,7 @@ sub null_schedule {
 # time a schedule should run, excluding today
 #
 # It takes a run_day argument (see check_today, above, for an explanation),
-# and returns an SQL date
+# and returns an integer, representing a number of days.
 sub get_next_date {
     my $day = shift;
 
@@ -689,11 +680,5 @@ sub get_next_date {
             $add_days += $daysinmonth[$now_month];
         }
     }
-
-    # Get a date in whatever format the database will accept
-    $sth = $dbh->prepare("SELECT CURRENT_DATE() + " .
-                         $dbh->sql_interval('? DAY'));
-    $sth->execute($add_days);
-    return $sth->fetch->[0];
+    return $add_days;
 }
-
